
from smart_module.module_system.module_skills import *
from smart_module.module_system.process_common import *

skill_name_pos = 1
skill_attribute_pos = 2
skill_max_level_pos= 3
skill_desc_pos = 4



def save_skills(export_dir):
  ofile = open(export_dir + "skills.txt","w",encoding="utf-8")
  ofile.write("%d\n"%(len(skills)))
  for i_skill in range(len(skills)):
    skill = skills[i_skill]
    ofile.write("skl_%s %s "%(skill[0], replace_spaces(skill[1])))
    ofile.write("%d %d %s\n"%(skill[skill_attribute_pos],skill[skill_max_level_pos],skill[skill_desc_pos].replace(" ","_")))
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_skills.py","w",encoding="utf-8")
  for i_skill in range(len(skills)):
    ofile.write("skl_%s = %d\n"%(skills[i_skill][0],i_skill))
  ofile.write("\n\n")
  ofile.close()


def processSkills(context):
  print( "Exporting skills...")
  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()
  save_python_header(src_dir)
  save_skills(export_dir)



from smart_module.module_system.module_info import *
from smart_module.module_system.module_sounds import *


def write_python_header(sounds,src_dir):
  file = open(src_dir + "ID_sounds.py","w",encoding="utf-8")
  for i_sound in range(len(sounds)):
    file.write("snd_%s = %d\n"%(sounds[i_sound][0],i_sound))
  file.write("\n\n")
  file.close()

def write_sounds(sound_samples, sounds,export_dir):
  ofile = open(export_dir + "sounds.txt","w",encoding="utf-8")
  ofile.write("soundsfile version 3\n")
  ofile.write("%d\n"%len(sound_samples))
  for sound_sample in sound_samples:
    ofile.write(" %s %d\n"%sound_sample)
  ofile.write("%d\n"%len(sounds))
  for sound in sounds:
    ofile.write("snd_%s %d %d "%(sound[0], sound[1],len(sound[2])))
    sample_list = sound[2]
    for s in sample_list:
      ofile.write("%d %d "%(s[0], s[1]))
    ofile.write("\n")
  ofile.close()

def compile_sounds(sounds):
  all_sounds = []
  for sound in sounds:
    sound_files = sound[2]
    sound_flags = sound[1]
    for i_sound_file in range(len(sound_files)):
      sound_file = sound_files[i_sound_file]
      if (type(sound_file) != type([])):
        sound_file = [sound_file, 0]
      sound_no = 0
      found = 0
      while (sound_no< (len(all_sounds))) and (not found):
        if all_sounds[sound_no][0] == sound_file[0]:
          found = 1
        else:
          sound_no += 1
      if not found:
        all_sounds.append((sound_file[0], sound_flags))
        sound_no = len(all_sounds) - 1
      sound_files[i_sound_file] = [sound_no, sound_file[1]]
  return all_sounds


def processSounds(context):
  print( "Exporting sounds...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  sound_samples = compile_sounds(sounds)
  write_sounds(sound_samples, sounds,export_dir)
  write_python_header(sounds,src_dir)



from smart_module.module_system.module_game_menus import *
from smart_module.module_system.process_operations import *


def save_game_menu_item(ofile,variable_list,variable_uses,menu_item,tag_uses,quick_strings):
  ofile.write(" mno_%s "%(menu_item[0]))
  save_statement_block(ofile,0, 1, menu_item[1], variable_list, variable_uses,tag_uses,quick_strings)
  ofile.write(" %s "%(menu_item[2].replace(" ","_")))
  save_statement_block(ofile,0, 1, menu_item[3], variable_list, variable_uses,tag_uses,quick_strings)
  door_name = "."
  if (len(menu_item) > 4):
    door_name = menu_item[4]
  ofile.write(" %s "%(door_name.replace(" ","_")))
    

def save_game_menus(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  ofile = open(export_dir + "menus.txt","w",encoding="utf-8")
  ofile.write("menusfile version 1\n")
  ofile.write(" %d\n"%(len(game_menus)))
  for game_menu in game_menus:
    ofile.write("menu_%s %d %s %s"%(game_menu[0],game_menu[1],game_menu[2].replace(" ","_"),game_menu[3]))
    save_statement_block(ofile,0,1, game_menu[4]  , variable_list, variable_uses,tag_uses,quick_strings)
    menu_items = game_menu[5]
    ofile.write("%d\n"%(len(menu_items)))
    for menu_item in menu_items:
      save_game_menu_item(ofile,variable_list,variable_uses,menu_item,tag_uses,quick_strings)
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_menus.py","w",encoding="utf-8")
  for i_game_menu in range(len(game_menus)):
    ofile.write("menu_%s = %d\n"%(game_menus[i_game_menu][0],i_game_menu))
  ofile.close()


def processGameMenus(context):
  print( "Exporting game menus data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_game_menus(variables, variable_uses, tag_uses, quick_strings,export_dir)
  save_variables(export_dir, variables, variable_uses)
  save_tag_uses(export_dir, tag_uses)
  save_quick_strings(export_dir, quick_strings)


###################################################
# header_dialogs.py
# This file contains declarations for dialogs
# DO NOT EDIT THIS FILE!
###################################################

speaker_pos = 0
ipt_token_pos = 1
sentence_conditions_pos = 2
text_pos = 3
opt_token_pos = 4
sentence_consequences_pos = 5


anyone      = 0x00000fff
repeat_for_factions = 0x00001000
repeat_for_parties  = 0x00002000
repeat_for_troops   = 0x00003000
repeat_for_100      = 0x00004000
repeat_for_1000     = 0x00005000

plyr                = 0x00010000
party_tpl           = 0x00020000
auto_proceed        = 0x00040000
multi_line          = 0x00080000

suf_other_bits   = 20

def other(other_troop_id):
  return other_troop_id << suf_other_bits




from smart_module.smart_core.banner_print import BannerPrinter
from smart_module.smart_core.base.executor.smart_module_international import SmartModuleLanguageExecutor
from smart_module.smart_core.base.executor.smart_modules_command_executor import CommandExecutor
from smart_module.smart_core.base.executor.smart_modules_runtime import SmartModuleExecutor


################################################
###
### 编译smart_module的主要入口
### 1.编译快捷命令
### 2.汉化
###
################################################
from smart_module.smart_core.smart_modules_log import info
from smart_module.smart_core.utils.DateUtil import DateUtil

def processSmartModule(context):
    modules = context.configParser.getSmartModules()
    bannerPrinter = context.bannerPrinter
    dateUtil = DateUtil()
    dateUtil.start()
    bannerPrinter.print()
    info("【编译】smartModule所有模块")
    ## 编译脚本
    commandExecutor = CommandExecutor()
    smartModuleExecutor = SmartModuleExecutor()
    for smartModule in modules:
        info("【指令收集】开始：{}".format(smartModule["name"]))
        commandList = smartModuleExecutor.collectCommandList(smartModule)
        commandExecutor.addCommands(commandList)
        info("【指令收集】完成：{}".format(smartModule["name"]))
    info("【执行指令】开始")
    commandExecutor.execCommand()
    info("【执行指令】完成")
    info("【编译】完成")

    ## 汉化文件
    smartModuleInternational = SmartModuleLanguageExecutor(context)
    smartModuleInternational.preprocessInternational(modules)

    info("SmartModule模块完成编译！！！")

    ##info("SmartModule模块编译花费时间：{}".format(dateUtil.deltaStr(dateUtil.getAndStart())))
    info("SmartModule模块编译花费时间：{}秒".format(round(dateUtil.getAndStart(),2)))



import os.path

from smart_module.module_system.module_info import *
from smart_module.module_system.module_strings import *

from smart_module.module_system.process_common import *


def save_strings(strings,export_dir):
  ofile = open(export_dir + "strings.txt","w",encoding="utf-8")
  ofile.write("stringsfile version 1\n")
  ofile.write("%d\n"%len(strings))
  for i_string in range(len(strings)):
    str = strings[i_string]
    ofile.write("str_%s %s\n"%(convert_to_identifier(str[0]),replace_spaces(str[1])))
  ofile.close()

def save_python_header(src_dir):
  ##print(os.path.abspath(src_dir + "ID_strings.py"))
  ofile = open(src_dir + "ID_strings.py","w",encoding="utf-8")
  for i_string in range(len(strings)):
    ofile.write("str_%s = %d\n"%(convert_to_identifier(strings[i_string][0]),i_string))
  ofile.write("\n\n")
  ofile.close()


def processStrings(context):
  print( "Exporting strings...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  save_strings(strings,export_dir)




import os
import sys

from smart_module.smart_core.base.exception import SmartModuleExceptin


################################################
###
### 配置解析器：解析器
### 1.支持多环境
### 2.提供默认配置
### 3.以后支持命令行（optparse）
###
################################################

class SmartModuleConfigParser:
    def __init__(self,config):
        self.config = config;
        self.profile = self._parseProfile()
        self.environments = self.config.get("environments")
        self.currentConfig = self.environments.get(self.profile)
        if self.currentConfig == None:
            raise SmartModuleExceptin("未找到环境配置，请检查：" + str(self.profile))

    def __str__(self):
        return "smart module 配置解析器"

    def _parseProfile(self):
        ## 环境配置
        profile = self.config.get('profile')

        if profile == None:
            ## 命令行参数
            for arg in sys.argv:
                if arg.startswith("profile="):
                    profile = arg[0:len("profile=")].strip()
                    break;

        if profile == None:
            ## 环境变量
            profile = os.getenv("profile")

        if profile == None:
            ## 如果都没有使用第一个
            environments =  self.config.get("environments")
            (env,config) = environments.items[0]
            profile = env
        return profile

    def getConfig(self,key,default=None):
        value = self.currentConfig.get(key)
        return value if value != None else default

    def getProfile(self):
        '''
            获得输出目录
        :return:
        '''
        return self.profile

    def getExportDir(self):
        '''
            获得输出目录
        :return:
        '''
        return self.getConfig("exportDir")

    def getSrcDir(self):
        '''
            获得源代（Native）码位置
            默认为：module_system
        :return:
        '''
        return self.getConfig("srcDir","./module_system//")

    def getSmartModules(self):
        '''
            获得全部smart模块
        :return:
        '''
        return self.getConfig("smartModules")

    def getSkipNative(self):
        '''
            是否跳过原始（Native）编译
            默认：不跳过
        :return:
        '''
        return self.getConfig("skipNative",False)


###################################################
# header_quests.py
# This file contains declarations for quests
# DO NOT EDIT THIS FILE!
###################################################

from smart_module.module_system.header_common import *

#quest flags
qf_show_progression      = 0x00000001
qf_random_quest          = 0x00000002




from smart_module.smart_modules.map.SurvivalInTheWild import initSurvivalTypes

if __name__ == '__main__':
    datas = initSurvivalTypes()

    gameMenuOptionList = datas.get("gameMenuOptionList")
    gameStringsList = datas.get("gameStringsList")


# -*- coding: UTF-8 -*-
import datetime
import time


class DateUtil:

    def __init__(self):
        self.begin = 0
        self.allCostTime = 0

    def getDateStr(self):
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())

    def getTime(self):
        return time.time()

    def start(self):
        self.begin = time.time();

    def getAndStart(self):
        currCost = time.time() - self.begin;
        self.begin = time.time();
        self.allCostTime = self.allCostTime + currCost;
        return currCost

    def getCostAndClear(self):
        allCostTime = self.allCostTime;
        self.allCostTime = 0;
        return allCostTime

    def deltaStr(self,delta):
        deltaObj = datetime.timedelta(seconds=delta)
        return str(deltaObj)



# dateUtil = DateUtil()
#
# print(dateUtil.getTime())


###################################################
# header_common.py
# This file contains common declarations.
# DO NOT EDIT THIS FILE!
###################################################

#client events
multiplayer_event_set_item_selection                          = 0
multiplayer_event_set_bot_selection                           = 1
multiplayer_event_admin_start_map                             = 2
multiplayer_event_admin_set_max_num_players                   = 3
multiplayer_event_admin_set_num_bots_in_team                  = 4
multiplayer_event_admin_set_friendly_fire                     = 5
multiplayer_event_admin_set_ghost_mode                        = 6
multiplayer_event_admin_set_control_block_dir                 = 7
multiplayer_event_admin_set_add_to_servers_list               = 8
multiplayer_event_admin_set_respawn_period                    = 9
multiplayer_event_admin_set_game_max_minutes                  = 10
multiplayer_event_admin_set_round_max_seconds                 = 11
multiplayer_event_admin_set_game_max_points                   = 12
multiplayer_event_admin_set_point_gained_from_flags           = 13
multiplayer_event_admin_set_point_gained_from_capturing_flag  = 14
multiplayer_event_admin_set_server_name                       = 15
multiplayer_event_admin_set_game_password                     = 16
multiplayer_event_admin_set_team_faction                      = 17
multiplayer_event_open_admin_panel                            = 18
multiplayer_event_change_team_no                              = 19
multiplayer_event_change_troop_id                             = 20
multiplayer_event_start_new_poll                              = 21
multiplayer_event_answer_to_poll                              = 22
multiplayer_event_admin_kick_player                           = 23
multiplayer_event_admin_ban_player                            = 24
multiplayer_event_admin_set_num_bots_voteable                 = 25
multiplayer_event_admin_set_factions_voteable                 = 26
multiplayer_event_admin_set_maps_voteable                     = 27
multiplayer_event_admin_set_player_respawn_as_bot             = 28
multiplayer_event_admin_set_combat_speed                      = 29
multiplayer_event_admin_set_respawn_count                     = 30
multiplayer_event_admin_set_kick_voteable                     = 31
multiplayer_event_admin_set_ban_voteable                      = 32
multiplayer_event_admin_set_valid_vote_ratio                  = 33
multiplayer_event_admin_set_auto_team_balance_limit           = 34
multiplayer_event_admin_set_welcome_message                   = 35
multiplayer_event_admin_set_initial_gold_multiplier           = 36
multiplayer_event_admin_set_battle_earnings_multiplier        = 37
multiplayer_event_admin_set_round_earnings_multiplier         = 38
multiplayer_event_admin_set_melee_friendly_fire               = 39
multiplayer_event_admin_set_friendly_fire_damage_self_ratio   = 40
multiplayer_event_admin_set_friendly_fire_damage_friend_ratio = 41
multiplayer_event_admin_set_allow_player_banners              = 42
multiplayer_event_admin_set_force_default_armor               = 43
multiplayer_event_admin_set_anti_cheat                        = 44
multiplayer_event_open_game_rules                             = 45
multiplayer_event_offer_duel                                  = 46
multiplayer_event_admin_set_disallow_ranged_weapons           = 47

#server events
multiplayer_event_return_max_num_players                      = 50
multiplayer_event_return_num_bots_in_team                     = 51
multiplayer_event_return_friendly_fire                        = 52
multiplayer_event_return_ghost_mode                           = 53
multiplayer_event_return_control_block_dir                    = 54
multiplayer_event_return_combat_speed                         = 55
multiplayer_event_return_add_to_servers_list                  = 56
multiplayer_event_return_respawn_period                       = 57
multiplayer_event_return_game_max_minutes                     = 58
multiplayer_event_return_round_max_seconds                    = 59
multiplayer_event_return_game_max_points                      = 60
multiplayer_event_return_point_gained_from_flags              = 61
multiplayer_event_return_point_gained_from_capturing_flag     = 62
multiplayer_event_return_server_name                          = 63
multiplayer_event_return_game_password                        = 64
multiplayer_event_return_game_type                            = 65
multiplayer_event_return_confirmation                         = 66
multiplayer_event_return_rejection                            = 67
multiplayer_event_show_multiplayer_message                    = 68
multiplayer_event_draw_this_round                             = 69
multiplayer_event_set_attached_scene_prop                     = 70
multiplayer_event_set_team_flag_situation                     = 71
multiplayer_event_set_team_score                              = 72
multiplayer_event_set_num_agents_around_flag                  = 73
multiplayer_event_ask_for_poll                                = 74
multiplayer_event_change_flag_owner                           = 75
multiplayer_event_use_item                                    = 76
multiplayer_event_set_scene_prop_open_or_close                = 77
multiplayer_event_set_round_start_time                        = 78
multiplayer_event_force_start_team_selection                  = 79
multiplayer_event_start_death_mode                            = 80
multiplayer_event_return_num_bots_voteable                    = 81
multiplayer_event_return_factions_voteable                    = 82
multiplayer_event_return_maps_voteable                        = 83
multiplayer_event_return_next_team_faction                    = 84
multiplayer_event_return_player_respawn_as_bot                = 85
multiplayer_event_set_player_score_kill_death                 = 86
multiplayer_event_set_day_time                                = 87
multiplayer_event_return_respawn_count                        = 88
multiplayer_event_return_player_respawn_spent                 = 89
multiplayer_event_return_kick_voteable                        = 90
multiplayer_event_return_ban_voteable                         = 91
multiplayer_event_return_valid_vote_ratio                     = 92
multiplayer_event_return_auto_team_balance_limit              = 93
multiplayer_event_return_initial_gold_multiplier              = 94
multiplayer_event_return_battle_earnings_multiplier           = 95
multiplayer_event_return_round_earnings_multiplier            = 96
multiplayer_event_return_renaming_server_allowed              = 97
multiplayer_event_return_changing_game_type_allowed           = 98
multiplayer_event_return_melee_friendly_fire                  = 99
multiplayer_event_return_friendly_fire_damage_self_ratio      = 100
multiplayer_event_return_friendly_fire_damage_friend_ratio    = 101
multiplayer_event_return_allow_player_banners                 = 102
multiplayer_event_return_force_default_armor                  = 103
multiplayer_event_return_anti_cheat                           = 104
multiplayer_event_return_open_game_rules                      = 105
multiplayer_event_return_max_num_bots                         = 106
multiplayer_event_return_server_mission_timer_while_player_joined = 107
multiplayer_event_show_duel_request                           = 108
multiplayer_event_start_duel                                  = 109
multiplayer_event_cancel_duel                                 = 110
multiplayer_event_show_server_message                         = 111
multiplayer_event_return_disallow_ranged_weapons              = 112

#multiplayer message types
multiplayer_message_type_auto_team_balance_done      = 2
multiplayer_message_type_auto_team_balance_next      = 3
multiplayer_message_type_capture_the_flag_score      = 4
multiplayer_message_type_flag_returned_home          = 5
multiplayer_message_type_capture_the_flag_stole      = 6
multiplayer_message_type_poll_result                 = 7
multiplayer_message_type_flag_neutralized            = 8
multiplayer_message_type_flag_captured               = 9
multiplayer_message_type_flag_is_pulling             = 10
multiplayer_message_type_auto_team_balance_no_need   = 11
multiplayer_message_type_round_result_in_battle_mode = 12
multiplayer_message_type_round_result_in_siege_mode  = 13
multiplayer_message_type_round_draw                  = 14
multiplayer_message_type_target_destroyed            = 15
multiplayer_message_type_defenders_saved_n_targets   = 16
multiplayer_message_type_attackers_won_the_round     = 17
multiplayer_message_type_start_death_mode            = 18

#multiplayer game type indices
multiplayer_game_type_deathmatch             = 0
multiplayer_game_type_team_deathmatch        = 1
multiplayer_game_type_battle                 = 2
multiplayer_game_type_destroy                = 3
multiplayer_game_type_capture_the_flag       = 4
multiplayer_game_type_headquarters           = 5
multiplayer_game_type_siege                  = 6
multiplayer_game_type_duel                   = 7
multiplayer_num_game_types                   = 8

#admin panel value ranges
multiplayer_round_max_seconds_min            = 60
multiplayer_round_max_seconds_max            = 901
multiplayer_respawn_period_min               = 3
multiplayer_respawn_period_max               = 31 #can only be 30 seconds max (due to agent deletion after that period)

multiplayer_siege_mod_defender_team_extra_respawn_time = 27 #It was 20 in 1.113 but it is increased it to 25 in 1.121 because defenders were mostly defeating enemy.
multiplayer_new_agents_finish_spawning_time = 30
multiplayer_max_possible_player_id = 1000

#team 1 and team 2 are 0 and 1 respectively
multi_team_spectator = 2
multi_team_unassigned = multi_team_spectator + 1

multi_data_maps_for_game_type_begin = 0
multi_data_maps_for_game_type_end = multi_data_maps_for_game_type_begin + 30
multi_data_troop_button_indices_begin = multi_data_maps_for_game_type_end
multi_data_troop_button_indices_end = multi_data_troop_button_indices_begin + 16 #maximum 16 troops per faction
multi_data_item_button_indices_begin = multi_data_troop_button_indices_end
multi_data_item_button_indices_end = multi_data_item_button_indices_begin + 100 #maximum 100 items per troop
multi_data_flag_owner_begin = multi_data_item_button_indices_end
multi_data_flag_owner_end = multi_data_flag_owner_begin + 10 #maximum of 10 flags per scene
multi_data_flag_players_around_begin = multi_data_flag_owner_end
multi_data_flag_players_around_end = multi_data_flag_players_around_begin + 10 #maximum of 10 flags per scene
multi_data_flag_owned_seconds_begin = multi_data_flag_players_around_end
multi_data_flag_owned_seconds_end = multi_data_flag_owned_seconds_begin + 10 #maximum of 10 flags per scene
multi_data_flag_pull_code_begin = multi_data_flag_owned_seconds_end
multi_data_flag_pull_code_end = multi_data_flag_pull_code_begin + 10 #maximum of 10 flags per scene
multi_data_player_index_list_begin = multi_data_flag_pull_code_end

#Entry points 100..109 is used for showing initial points for moveable and usable scene props like siege ladder.
multi_entry_points_for_usable_items_start = 100
multi_entry_points_for_usable_items_end = multi_entry_points_for_usable_items_start + 10


#multi_item_class_type_other = 0
multi_item_class_type_sword = 1
multi_item_class_type_axe = 2
multi_item_class_type_blunt = 3
multi_item_class_type_war_picks = 4
multi_item_class_type_cleavers = 5
multi_item_class_type_two_handed_sword = 6
multi_item_class_type_two_handed_axe = 7
multi_item_class_type_spear = 8
multi_item_class_type_lance = 9
multi_item_class_type_small_shield = 10
multi_item_class_type_large_shield = 11
multi_item_class_type_bow = 12
multi_item_class_type_crossbow = 13
multi_item_class_type_arrow = 14
multi_item_class_type_bolt = 15
multi_item_class_type_throwing = 16
multi_item_class_type_throwing_axe = 17
multi_item_class_type_horse = 18
multi_item_class_type_light_armor = 19
multi_item_class_type_medium_armor = 20
multi_item_class_type_heavy_armor = 21
multi_item_class_type_light_helm = 22
multi_item_class_type_heavy_helm = 23
multi_item_class_type_light_foot = 24
multi_item_class_type_heavy_foot = 25
multi_item_class_type_glove = 26

multi_item_class_type_melee_weapons_begin = multi_item_class_type_sword
multi_item_class_type_melee_weapons_end = multi_item_class_type_small_shield
multi_item_class_type_ranged_weapons_begin = multi_item_class_type_bow
multi_item_class_type_ranged_weapons_end = multi_item_class_type_horse
multi_item_class_type_shields_begin = multi_item_class_type_melee_weapons_end
multi_item_class_type_shields_end = multi_item_class_type_bow

multi_item_class_type_weapons_begin = multi_item_class_type_sword
multi_item_class_type_weapons_end = multi_item_class_type_horse
multi_item_class_type_horses_begin = multi_item_class_type_weapons_end
multi_item_class_type_horses_end = multi_item_class_type_light_armor
multi_item_class_type_bodies_begin = multi_item_class_type_horses_end
multi_item_class_type_bodies_end = multi_item_class_type_light_helm
multi_item_class_type_heads_begin = multi_item_class_type_bodies_end
multi_item_class_type_heads_end = multi_item_class_type_light_foot
multi_item_class_type_feet_begin = multi_item_class_type_heads_end
multi_item_class_type_feet_end = multi_item_class_type_glove
multi_item_class_type_gloves_begin = multi_item_class_type_feet_end
multi_item_class_type_gloves_end = multi_item_class_type_glove + 1

multi_troop_class_other = 0
multi_troop_class_infantry = 1
multi_troop_class_spearman = 2
multi_troop_class_cavalry = 3
multi_troop_class_archer = 4
multi_troop_class_crossbowman = 5
multi_troop_class_mounted_archer = 6
multi_troop_class_mounted_crossbowman = 7

multi_num_valid_entry_points = 64
multi_num_valid_entry_points_div_2 = 32

#normal money management system
multi_battle_round_team_money_add = 500
multi_destroy_save_or_destroy_target_money_add = 100
multi_destroy_target_money_add = 1500
multi_initial_gold_value = 1000
multi_max_gold_that_can_be_stored = 15000

multi_killer_agent_standard_money_add = 150 #(2/3 = 100 for battle & destroy, 3/3 = 150 for siege, 4/3 = 200 for deathmatch/team deathmatch/capture the flag/headquarters)
multi_killer_agent_loot_percentage_share = 12 #(2/3 = 8% for battle & destroy, 3/3 = 12% for siege, 4/3 = 16% for deathmatch/team deathmatch/capture the flag/headquarters)
multi_dead_agent_loot_percentage_share = 48 #(2/3 = 32% for battle & destroy, 3/3 = 48% for siege, 4/3 = 64% for deathmatch/team deathmatch/capture the flag/headquarters)
multi_minimum_gold = 1000 #(same in all modes)

multi_minimum_target_health = 1200

multi_max_seconds_flag_can_stay_in_ground = 60

multi_capture_the_flag_score_flag_returning = 1

multi_initial_spawn_point_team_1 = 0
multi_initial_spawn_point_team_2 = 32
multi_base_point_team_1 = 64
multi_base_point_team_2 = 65
multi_siege_flag_point = 66
multi_death_mode_point = 67

multi_headquarters_pole_height = 900
multi_headquarters_flag_height_to_win = 800 #used in sd death mode
multi_headquarters_flag_initial_height = 100 #used in sd death mode
multi_headquarters_max_distance_sq_to_raise_flags = 1600 #4m * 4m * 100 = 1600
multi_headquarters_distance_sq_to_set_flag = 8100 #9m * 9m * 100 = 8100
multi_headquarters_distance_sq_to_change_flag = 400 #2m * 2m * 100 = 400
multi_headquarters_distance_to_change_flag = 200 #2m * 100 = 200
multi_distance_sq_to_use_belfry = 36 #6m * 6m = 36 (there is no * 100 for this one because it uses get_sq_distance_between_positions_in_meters instead of get_sq_distance_between_positions)
multi_max_sq_dist_between_agents_to_longer_mof_time = 49 #7m * 7m = 49m
min_allowed_flag_height_difference_to_make_score = 50

#these two values are about when master of field will be kicked
multiplayer_battle_formula_value_a = 15
multiplayer_battle_formula_value_b = 18000 #think about 18000-20000 if death mod very much happens.

multiplayer_spawn_above_opt_enemy_dist_point = 32 #while finding most suitable spawn point if nearest enemy is further than 32 meters give negative points to that spawn point
multiplayer_spawn_min_enemy_dist_limit = 45 #while finding most suitable spawn point if nearest enemy is closer than 45 meters give negative points to that spawn point, (squared increase)

multiplayer_poll_disable_period = 900 #15 minutes

#menu variables
escape_menu_item_height = 40



bignum = 0x40000000000000000000000000000000

op_num_value_bits = 24 + 32

tag_register        =  1
tag_variable        =  2
tag_string          =  3
tag_item            =  4
tag_troop           =  5
tag_faction         =  6
tag_quest           =  7
tag_party_tpl       =  8
tag_party           =  9
tag_scene           = 10
tag_mission_tpl     = 11
tag_menu            = 12
tag_script          = 13
tag_particle_sys    = 14
tag_scene_prop      = 15
tag_sound           = 16
tag_local_variable  = 17
tag_map_icon        = 18
tag_skill           = 19
tag_mesh            = 20
tag_presentation    = 21
tag_quick_string    = 22
tag_track	    = 23
tag_tableau         = 24
tag_animation       = 25
tags_end            = 26


opmask_register             =  tag_register       << op_num_value_bits
opmask_variable             =  tag_variable       << op_num_value_bits
##opmask_string             =  tag_string         << op_num_value_bits
##opmask_item_index         =  tag_item           << op_num_value_bits
##opmask_troop_index        =  tag_troop          << op_num_value_bits
##opmask_faction_index      =  tag_faction        << op_num_value_bits
opmask_quest_index          =  tag_quest          << op_num_value_bits
##opmask_p_template_index   =  tag_party_tpl      << op_num_value_bits
##opmask_party_index        =  tag_party          << op_num_value_bits
##opmask_scene_index        =  tag_scene          << op_num_value_bits
##opmask_mission_tpl_index  =  tag_mission_tpl    << op_num_value_bits
##opmask_menu_index         =  tag_menu           << op_num_value_bits
##opmask_script             =  tag_script         << op_num_value_bits
##opmask_particle_sys       =  tag_particle_sys   << op_num_value_bits
##opmask_scene_prop         =  tag_scene_prop     << op_num_value_bits
##opmask_sound              =  tag_sound          << op_num_value_bits
##opmask_map_icon           =  tag_map_icon       << op_num_value_bits
opmask_local_variable       =  tag_local_variable << op_num_value_bits
opmask_quick_string         =  tag_quick_string   << op_num_value_bits


def reg(reg_no):
  if (reg_no < 0):
    raise RuntimeError("Error register_no negative")
  return opmask_register | reg_no

def find_object(objects,object_id):
  result = -1
  num_objects = len(objects)
  i_object = 0
  object_id_lowercase = object_id.lower()
  while (i_object < num_objects) and (result == -1):
    object = objects[i_object]
    if (object[0].lower() == object_id_lowercase):
      result = i_object
    i_object += 1
  return result

s0  =  0
s1  =  1
s2  =  2
s3  =  3
s4  =  4
s5  =  5
s6  =  6
s7  =  7
s8  =  8
s9  =  9
s10 = 10
s11 = 11
s12 = 12
s13 = 13
s14 = 14
s15 = 15
s16 = 16
s17 = 17
s18 = 18
s19 = 19
s20 = 20
s21 = 21
s22 = 22
s23 = 23
s24 = 24
s25 = 25
s26 = 26
s27 = 27
s28 = 28
s29 = 29
s30 = 30
s31 = 31
s32 = 32
s33 = 33
s34 = 34
s35 = 35
s36 = 36
s37 = 37
s38 = 38
s39 = 39
s40 = 40
s41 = 41
s42 = 42
s43 = 43
s44 = 44
s45 = 45
s46 = 46
s47 = 47
s48 = 48
s49 = 49
s50 = 50
s51 = 51
s52 = 52
s53 = 53
s54 = 54
s55 = 55
s56 = 56
s57 = 57
s58 = 58
s59 = 59
s60 = 60
s61 = 61
s62 = 62
s63 = 63

s64 = 64
s65 = 65
s66 = 66
s67 = 67


pos0  =  0
pos1  =  1
pos2  =  2
pos3  =  3
pos4  =  4
pos5  =  5
pos6  =  6
pos7  =  7
pos8  =  8
pos9  =  9
pos10 = 10
pos11 = 11
pos12 = 12
pos13 = 13
pos14 = 14
pos15 = 15
pos16 = 16
pos17 = 17
pos18 = 18
pos19 = 19
pos20 = 20
pos21 = 21
pos22 = 22
pos23 = 23
pos24 = 24
pos25 = 25
pos26 = 26
pos27 = 27
pos28 = 28
pos29 = 29
pos30 = 30
pos31 = 31
pos32 = 32
pos33 = 33
pos34 = 34
pos35 = 35
pos36 = 36
pos37 = 37
pos38 = 38
pos39 = 39
pos40 = 40
pos41 = 41
pos42 = 42
pos43 = 43
pos44 = 44
pos45 = 45
pos46 = 46
pos47 = 47
pos48 = 48
pos49 = 49
pos50 = 50
pos51 = 51
pos52 = 52
pos53 = 53
pos54 = 54
pos55 = 55
pos56 = 56
pos57 = 57
pos58 = 58
pos59 = 59
pos60 = 60
pos61 = 61
pos62 = 62
pos63 = 63
pos_belfry_begin = 64

reg0   = opmask_register| 0
reg1   = opmask_register| 1
reg2   = opmask_register| 2
reg3   = opmask_register| 3
reg4   = opmask_register| 4
reg5   = opmask_register| 5
reg6   = opmask_register| 6
reg7   = opmask_register| 7
reg8   = opmask_register| 8
reg9   = opmask_register| 9
reg10  = opmask_register|10
reg11  = opmask_register|11
reg12  = opmask_register|12
reg13  = opmask_register|13
reg14  = opmask_register|14
reg15  = opmask_register|15
reg16  = opmask_register|16
reg17  = opmask_register|17
reg18  = opmask_register|18
reg19  = opmask_register|19
reg20  = opmask_register|20
reg21  = opmask_register|21
reg22  = opmask_register|22
reg23  = opmask_register|23
reg24  = opmask_register|24
reg25  = opmask_register|25
reg26  = opmask_register|26
reg27  = opmask_register|27
reg28  = opmask_register|28
reg29  = opmask_register|29
reg30  = opmask_register|30
reg31  = opmask_register|31
reg32  = opmask_register|32
reg33  = opmask_register|33
reg34  = opmask_register|34
reg35  = opmask_register|35
reg36  = opmask_register|36
reg37  = opmask_register|37
reg38  = opmask_register|38
reg39  = opmask_register|39
reg40  = opmask_register|40
reg41  = opmask_register|41
reg42  = opmask_register|42
reg43  = opmask_register|43
reg44  = opmask_register|44
reg45  = opmask_register|45
reg46  = opmask_register|46
reg47  = opmask_register|47
reg48  = opmask_register|48
reg49  = opmask_register|49
reg50  = opmask_register|50
reg51  = opmask_register|51
reg52  = opmask_register|52
reg53  = opmask_register|53
reg54  = opmask_register|54
reg55  = opmask_register|55
reg56  = opmask_register|56
reg57  = opmask_register|57
reg58  = opmask_register|58
reg59  = opmask_register|59
reg60  = opmask_register|60
reg61  = opmask_register|61
reg62  = opmask_register|62
reg63  = opmask_register|63

reg65  = opmask_register|65

spf_all_teams_are_enemy                      = 0x00000001,
spf_is_horseman                              = 0x00000002,
spf_examine_all_spawn_points                 = 0x00000004,
spf_team_0_spawn_far_from_entry_32           = 0x00000008,
spf_team_1_spawn_far_from_entry_0            = 0x00000010,
spf_team_1_spawn_far_from_entry_66           = 0x00000020,
spf_team_0_spawn_near_entry_0                = 0x00000040,
spf_team_0_spawn_near_entry_66               = 0x00000080,
spf_team_1_spawn_near_entry_32               = 0x00000100,
spf_team_0_walkers_spawn_at_high_points      = 0x00000200,
spf_team_1_walkers_spawn_at_high_points      = 0x00000400,
spf_try_to_spawn_close_to_at_least_one_enemy = 0x00000800,
spf_care_agent_to_agent_distances_less       = 0x00001000,



pt_none = 0
pt_rescued_prisoners = 1
pt_enemy = 2
pt_hero_party = 3
pt_village_defenders = 4
pt_cattle_herd = 5
pt_looters = 6
pt_manhunters = 7
pt_steppe_bandits = 8
pt_taiga_bandits = 9
pt_desert_bandits = 10
pt_forest_bandits = 11
pt_mountain_bandits = 12
pt_sea_raiders = 13
pt_deserters = 14
pt_merchant_caravan = 15
pt_troublesome_bandits = 16
pt_bandits_awaiting_ransom = 17
pt_kidnapped_girl = 18
pt_village_farmers = 19
pt_spy_partners = 20
pt_runaway_serfs = 21
pt_spy = 22
pt_sacrificed_messenger = 23
pt_forager_party = 24
pt_scout_party = 25
pt_patrol_party = 26
pt_messenger_party = 27
pt_raider_party = 28
pt_raider_captives = 29
pt_kingdom_caravan_party = 30
pt_prisoner_train_party = 31
pt_default_prisoners = 32
pt_routed_warriors = 33
pt_center_reinforcements = 34
pt_kingdom_hero_party = 35
pt_kingdom_1_reinforcements_a = 36
pt_kingdom_1_reinforcements_b = 37
pt_kingdom_1_reinforcements_c = 38
pt_kingdom_2_reinforcements_a = 39
pt_kingdom_2_reinforcements_b = 40
pt_kingdom_2_reinforcements_c = 41
pt_kingdom_3_reinforcements_a = 42
pt_kingdom_3_reinforcements_b = 43
pt_kingdom_3_reinforcements_c = 44
pt_kingdom_4_reinforcements_a = 45
pt_kingdom_4_reinforcements_b = 46
pt_kingdom_4_reinforcements_c = 47
pt_kingdom_5_reinforcements_a = 48
pt_kingdom_5_reinforcements_b = 49
pt_kingdom_5_reinforcements_c = 50
pt_kingdom_6_reinforcements_a = 51
pt_kingdom_6_reinforcements_b = 52
pt_kingdom_6_reinforcements_c = 53
pt_steppe_bandit_lair = 54
pt_taiga_bandit_lair = 55
pt_desert_bandit_lair = 56
pt_forest_bandit_lair = 57
pt_mountain_bandit_lair = 58
pt_sea_raider_lair = 59
pt_looter_lair = 60
pt_bandit_lair_templates_end = 61
pt_leaded_looters = 62



###################################################
# header_scene_props.py
# This file contains declarations for scene props
# DO NOT EDIT THIS FILE!
###################################################

sokf_type_container        = 0x0000000000000005
sokf_type_ai_limiter       = 0x0000000000000008
sokf_type_barrier          = 0x0000000000000009
sokf_type_barrier_leave    = 0x000000000000000a
sokf_type_ladder           = 0x000000000000000b
sokf_type_barrier3d        = 0x000000000000000c
sokf_type_player_limiter   = 0x000000000000000d
sokf_type_mask             = 0x00000000000000FF
sokf_add_fire              = 0x0000000000000100
sokf_add_smoke             = 0x0000000000000200
sokf_add_light             = 0x0000000000000400
sokf_show_hit_point_bar    = 0x0000000000000800
sokf_place_at_origin       = 0x0000000000001000
sokf_dynamic               = 0x0000000000002000
sokf_invisible             = 0x0000000000004000
sokf_destructible          = 0x0000000000008000
sokf_moveable              = 0x0000000000010000
sokf_face_player           = 0x0000000000020000
sokf_dynamic_physics       = 0x0000000000040000
sokf_missiles_not_attached = 0x0000000000080000 #works only for dynamic mission objects
sokf_enforce_shadows       = 0x0000000000100000
sokf_dont_move_agent_over  = 0x0000000000200000
sokf_handle_as_flora       = 0x0000000001000000
sokf_static_movement       = 0x0000000002000000


spbf_hit_points_mask       = 0x00000000000000FF
spbf_hit_points_bits       = 20
spbf_init_use_time_mask    = 0x00000000000000FF
spbf_use_time_bits         = 28

def spr_hit_points(x):
  return ((x & spbf_hit_points_mask) << spbf_hit_points_bits)

def get_spr_hit_points(y):
  return (y >> spbf_hit_points_bits) & spbf_hit_points_mask

def spr_use_time(x):
  return ((x & spbf_init_use_time_mask) << spbf_use_time_bits)

def get_spr_use_time(y):
  return (y >> spbf_use_time_bits) & spbf_init_use_time_mask

#Animation Mode

spanim_linear              = 0
spanim_loop_linear         = 4



rt_water = 0
rt_mountain = 1
rt_steppe = 2
rt_plain = 3
rt_snow = 4
rt_desert = 5
rt_bridge = 7
rt_river  = 8
rt_mountain_forest = 9
rt_steppe_forest = 10
rt_forest = 11
rt_snow_forest = 12
rt_desert_forest = 13



spr_invalid_object = 0
spr_inventory = 1
spr_empty = 2
spr_chest_a = 3
spr_container_small_chest = 4
spr_container_chest_b = 5
spr_container_chest_c = 6
spr_player_chest = 7
spr_locked_player_chest = 8
spr_light_sun = 9
spr_light = 10
spr_light_red = 11
spr_light_night = 12
spr_torch = 13
spr_torch_night = 14
spr_barrier_20m = 15
spr_barrier_16m = 16
spr_barrier_8m = 17
spr_barrier_4m = 18
spr_barrier_2m = 19
spr_exit_4m = 20
spr_exit_8m = 21
spr_exit_16m = 22
spr_ai_limiter_2m = 23
spr_ai_limiter_4m = 24
spr_ai_limiter_8m = 25
spr_ai_limiter_16m = 26
spr_Shield = 27
spr_shelves = 28
spr_table_tavern = 29
spr_table_castle_a = 30
spr_chair_castle_a = 31
spr_pillow_a = 32
spr_pillow_b = 33
spr_pillow_c = 34
spr_interior_castle_g_square_keep_b = 35
spr_carpet_with_pillows_a = 36
spr_carpet_with_pillows_b = 37
spr_table_round_a = 38
spr_table_round_b = 39
spr_fireplace_b = 40
spr_fireplace_c = 41
spr_sofa_a = 42
spr_sofa_b = 43
spr_ewer_a = 44
spr_end_table_a = 45
spr_fake_houses_steppe_a = 46
spr_fake_houses_steppe_b = 47
spr_fake_houses_steppe_c = 48
spr_boat_destroy = 49
spr_destroy_house_a = 50
spr_destroy_house_b = 51
spr_destroy_house_c = 52
spr_destroy_heap = 53
spr_destroy_castle_a = 54
spr_destroy_castle_b = 55
spr_destroy_castle_c = 56
spr_destroy_castle_d = 57
spr_destroy_windmill = 58
spr_destroy_tree_a = 59
spr_destroy_tree_b = 60
spr_destroy_bridge_a = 61
spr_destroy_bridge_b = 62
spr_catapult = 63
spr_catapult_destructible = 64
spr_broom = 65
spr_garlic = 66
spr_garlic_b = 67
spr_destroy_a = 68
spr_destroy_b = 69
spr_bridge_wooden = 70
spr_bridge_wooden_snowy = 71
spr_grave_a = 72
spr_village_house_e = 73
spr_village_house_f = 74
spr_village_house_g = 75
spr_village_house_h = 76
spr_village_house_i = 77
spr_village_house_j = 78
spr_village_wall_a = 79
spr_village_wall_b = 80
spr_village_snowy_house_a = 81
spr_village_snowy_house_b = 82
spr_village_snowy_house_c = 83
spr_village_snowy_house_d = 84
spr_village_snowy_house_e = 85
spr_village_snowy_house_f = 86
spr_town_house_steppe_a = 87
spr_town_house_steppe_b = 88
spr_town_house_steppe_c = 89
spr_town_house_steppe_d = 90
spr_town_house_steppe_e = 91
spr_town_house_steppe_f = 92
spr_town_house_steppe_g = 93
spr_town_house_steppe_h = 94
spr_town_house_steppe_i = 95
spr_carpet_a = 96
spr_carpet_b = 97
spr_carpet_c = 98
spr_carpet_d = 99
spr_carpet_e = 100
spr_carpet_f = 101
spr_awning_a = 102
spr_awning_b = 103
spr_awning_c = 104
spr_awning_long = 105
spr_awning_long_b = 106
spr_awning_d = 107
spr_ship = 108
spr_ship_b = 109
spr_ship_c = 110
spr_ship_d = 111
spr_snowy_barrel_a = 112
spr_snowy_fence = 113
spr_snowy_wood_heap = 114
spr_village_snowy_stable_a = 115
spr_village_straw_house_a = 116
spr_village_stable_a = 117
spr_village_shed_a = 118
spr_village_shed_b = 119
spr_dungeon_door_cell_a = 120
spr_dungeon_door_cell_b = 121
spr_dungeon_door_entry_a = 122
spr_dungeon_door_entry_b = 123
spr_dungeon_door_entry_c = 124
spr_dungeon_door_direction_a = 125
spr_dungeon_door_direction_b = 126
spr_dungeon_door_stairs_a = 127
spr_dungeon_door_stairs_b = 128
spr_dungeon_bed_a = 129
spr_dungeon_bed_b = 130
spr_torture_tool_a = 131
spr_torture_tool_b = 132
spr_torture_tool_c = 133
spr_skeleton_head = 134
spr_skeleton_bone = 135
spr_skeleton_a = 136
spr_dungeon_stairs_a = 137
spr_dungeon_stairs_b = 138
spr_dungeon_torture_room_a = 139
spr_dungeon_entry_a = 140
spr_dungeon_entry_b = 141
spr_dungeon_entry_c = 142
spr_dungeon_cell_a = 143
spr_dungeon_cell_b = 144
spr_dungeon_cell_c = 145
spr_dungeon_corridor_a = 146
spr_dungeon_corridor_b = 147
spr_dungeon_corridor_c = 148
spr_dungeon_corridor_d = 149
spr_dungeon_direction_a = 150
spr_dungeon_direction_b = 151
spr_dungeon_room_a = 152
spr_dungeon_tower_stairs_a = 153
spr_dungeon_tower_cell_a = 154
spr_tunnel_a = 155
spr_tunnel_salt = 156
spr_salt_a = 157
spr_door_destructible = 158
spr_tutorial_door_a = 159
spr_tutorial_door_b = 160
spr_tutorial_flag_yellow = 161
spr_tutorial_flag_red = 162
spr_tutorial_flag_blue = 163
spr_interior_prison_a = 164
spr_interior_prison_b = 165
spr_interior_prison_cell_a = 166
spr_interior_prison_d = 167
spr_arena_archery_target_a = 168
spr_archery_butt_a = 169
spr_archery_target_with_hit_a = 170
spr_dummy_a = 171
spr_band_a = 172
spr_arena_sign = 173
spr_castle_h_battlement_a = 174
spr_castle_h_battlement_b = 175
spr_castle_h_battlement_c = 176
spr_castle_h_battlement_a2 = 177
spr_castle_h_battlement_b2 = 178
spr_castle_h_corner_a = 179
spr_castle_h_corner_c = 180
spr_castle_h_stairs_a = 181
spr_castle_h_stairs_b = 182
spr_castle_h_gatehouse_a = 183
spr_castle_h_keep_a = 184
spr_castle_h_keep_b = 185
spr_castle_h_house_a = 186
spr_castle_h_house_b = 187
spr_castle_h_house_c = 188
spr_castle_h_battlement_barrier = 189
spr_full_keep_b = 190
spr_castle_f_keep_a = 191
spr_castle_f_battlement_a = 192
spr_castle_f_battlement_a_destroyed = 193
spr_castle_f_battlement_b = 194
spr_castle_f_battlement_c = 195
spr_castle_f_battlement_d = 196
spr_castle_f_battlement_e = 197
spr_castle_f_sally_port_elevation = 198
spr_castle_f_battlement_corner_a = 199
spr_castle_f_battlement_corner_b = 200
spr_castle_f_battlement_corner_c = 201
spr_castle_f_door_a = 202
spr_castle_f_doors_top_a = 203
spr_castle_f_sally_door_a = 204
spr_castle_f_stairs_a = 205
spr_castle_f_tower_a = 206
spr_castle_f_wall_stairs_a = 207
spr_castle_f_wall_stairs_b = 208
spr_castle_f_wall_way_a = 209
spr_castle_f_wall_way_b = 210
spr_castle_f_gatehouse_a = 211
spr_castle_g_battlement_a = 212
spr_castle_g_battlement_a1 = 213
spr_castle_g_battlement_c = 214
spr_castle_g_corner_a = 215
spr_castle_g_corner_c = 216
spr_castle_g_tower_a = 217
spr_castle_g_gate_house = 218
spr_castle_g_gate_house_door_a = 219
spr_castle_g_gate_house_door_b = 220
spr_castle_g_square_keep_a = 221
spr_castle_i_battlement_a = 222
spr_castle_i_battlement_a1 = 223
spr_castle_i_battlement_c = 224
spr_castle_i_corner_a = 225
spr_castle_i_corner_c = 226
spr_castle_i_tower_a = 227
spr_castle_i_gate_house = 228
spr_castle_i_gate_house_door_a = 229
spr_castle_i_gate_house_door_b = 230
spr_castle_i_square_keep_a = 231
spr_mosque_a = 232
spr_stone_minaret_a = 233
spr_stone_house_a = 234
spr_stone_house_b = 235
spr_stone_house_c = 236
spr_stone_house_d = 237
spr_stone_house_e = 238
spr_stone_house_f = 239
spr_banner_pole = 240
spr_custom_banner_01 = 241
spr_custom_banner_02 = 242
spr_banner_a = 243
spr_banner_b = 244
spr_banner_c = 245
spr_banner_d = 246
spr_banner_e = 247
spr_banner_f = 248
spr_banner_g = 249
spr_banner_h = 250
spr_banner_i = 251
spr_banner_j = 252
spr_banner_k = 253
spr_banner_l = 254
spr_banner_m = 255
spr_banner_n = 256
spr_banner_o = 257
spr_banner_p = 258
spr_banner_q = 259
spr_banner_r = 260
spr_banner_s = 261
spr_banner_t = 262
spr_banner_u = 263
spr_banner_ba = 264
spr_banner_bb = 265
spr_banner_bc = 266
spr_banner_bd = 267
spr_banner_be = 268
spr_banner_bf = 269
spr_banner_bg = 270
spr_banner_bh = 271
spr_banner_bi = 272
spr_banner_bj = 273
spr_banner_bk = 274
spr_banner_bl = 275
spr_banner_bm = 276
spr_banner_bn = 277
spr_banner_bo = 278
spr_banner_bp = 279
spr_banner_bq = 280
spr_banner_br = 281
spr_banner_bs = 282
spr_banner_bt = 283
spr_banner_bu = 284
spr_banner_ca = 285
spr_banner_cb = 286
spr_banner_cc = 287
spr_banner_cd = 288
spr_banner_ce = 289
spr_banner_cf = 290
spr_banner_cg = 291
spr_banner_ch = 292
spr_banner_ci = 293
spr_banner_cj = 294
spr_banner_ck = 295
spr_banner_cl = 296
spr_banner_cm = 297
spr_banner_cn = 298
spr_banner_co = 299
spr_banner_cp = 300
spr_banner_cq = 301
spr_banner_cr = 302
spr_banner_cs = 303
spr_banner_ct = 304
spr_banner_cu = 305
spr_banner_da = 306
spr_banner_db = 307
spr_banner_dc = 308
spr_banner_dd = 309
spr_banner_de = 310
spr_banner_df = 311
spr_banner_dg = 312
spr_banner_dh = 313
spr_banner_di = 314
spr_banner_dj = 315
spr_banner_dk = 316
spr_banner_dl = 317
spr_banner_dm = 318
spr_banner_dn = 319
spr_banner_do = 320
spr_banner_dp = 321
spr_banner_dq = 322
spr_banner_dr = 323
spr_banner_ds = 324
spr_banner_dt = 325
spr_banner_du = 326
spr_banner_ea = 327
spr_banner_eb = 328
spr_banner_ec = 329
spr_banner_ed = 330
spr_banner_ee = 331
spr_banner_ef = 332
spr_banner_eg = 333
spr_banner_eh = 334
spr_banner_ei = 335
spr_banner_ej = 336
spr_banner_ek = 337
spr_banner_el = 338
spr_banner_em = 339
spr_banner_en = 340
spr_banner_eo = 341
spr_banner_ep = 342
spr_banner_eq = 343
spr_banner_er = 344
spr_banner_es = 345
spr_banner_et = 346
spr_banner_eu = 347
spr_banner_f01 = 348
spr_banner_f02 = 349
spr_banner_f03 = 350
spr_banner_f04 = 351
spr_banner_f05 = 352
spr_banner_f06 = 353
spr_banner_f07 = 354
spr_banner_f08 = 355
spr_banner_f09 = 356
spr_banner_f10 = 357
spr_banner_f11 = 358
spr_banner_f12 = 359
spr_banner_f13 = 360
spr_banner_f14 = 361
spr_banner_f15 = 362
spr_banner_f16 = 363
spr_banner_f17 = 364
spr_banner_f18 = 365
spr_banner_f19 = 366
spr_banner_f20 = 367
spr_banner_g01 = 368
spr_banner_g02 = 369
spr_banner_g03 = 370
spr_banner_g04 = 371
spr_banner_g05 = 372
spr_banner_g06 = 373
spr_banner_g07 = 374
spr_banner_g08 = 375
spr_banner_g09 = 376
spr_banner_g10 = 377
spr_banner_kingdom_a = 378
spr_banner_kingdom_b = 379
spr_banner_kingdom_c = 380
spr_banner_kingdom_d = 381
spr_banner_kingdom_e = 382
spr_banner_kingdom_f = 383
spr_banner_f21 = 384
spr_tavern_chair_a = 385
spr_tavern_chair_b = 386
spr_tavern_table_a = 387
spr_tavern_table_b = 388
spr_fireplace_a = 389
spr_barrel = 390
spr_bench_tavern = 391
spr_bench_tavern_b = 392
spr_bowl_wood = 393
spr_chandelier_table = 394
spr_chandelier_tavern = 395
spr_chest_gothic = 396
spr_chest_b = 397
spr_chest_c = 398
spr_counter_tavern = 399
spr_cup = 400
spr_dish_metal = 401
spr_gothic_chair = 402
spr_gothic_stool = 403
spr_grate = 404
spr_jug = 405
spr_potlamp = 406
spr_weapon_rack = 407
spr_weapon_rack_big = 408
spr_tavern_barrel = 409
spr_tavern_barrel_b = 410
spr_merchant_sign = 411
spr_tavern_sign = 412
spr_sack = 413
spr_skull_a = 414
spr_skull_b = 415
spr_skull_c = 416
spr_skull_d = 417
spr_skeleton_cow = 418
spr_cupboard_a = 419
spr_box_a = 420
spr_bucket_a = 421
spr_straw_a = 422
spr_straw_b = 423
spr_straw_c = 424
spr_cloth_a = 425
spr_cloth_b = 426
spr_mat_a = 427
spr_mat_b = 428
spr_mat_c = 429
spr_mat_d = 430
spr_wood_a = 431
spr_wood_b = 432
spr_wood_heap = 433
spr_wood_heap_b = 434
spr_water_well_a = 435
spr_net_a = 436
spr_net_b = 437
spr_meat_hook = 438
spr_cooking_pole = 439
spr_bowl_a = 440
spr_bucket_b = 441
spr_washtub_a = 442
spr_washtub_b = 443
spr_table_trunk_a = 444
spr_chair_trunk_a = 445
spr_chair_trunk_b = 446
spr_chair_trunk_c = 447
spr_table_trestle_long = 448
spr_table_trestle_small = 449
spr_chair_trestle = 450
spr_wheel = 451
spr_ladder = 452
spr_cart = 453
spr_village_stand = 454
spr_wooden_stand = 455
spr_table_small = 456
spr_table_small_b = 457
spr_small_timber_frame_house_a = 458
spr_timber_frame_house_b = 459
spr_timber_frame_house_c = 460
spr_timber_frame_extension_a = 461
spr_timber_frame_extension_b = 462
spr_stone_stairs_a = 463
spr_stone_stairs_b = 464
spr_railing_a = 465
spr_side_building_a = 466
spr_battlement_a = 467
spr_battlement_a_destroyed = 468
spr_round_tower_a = 469
spr_small_round_tower_a = 470
spr_small_round_tower_roof_a = 471
spr_square_keep_a = 472
spr_square_tower_roof_a = 473
spr_gate_house_a = 474
spr_gate_house_b = 475
spr_small_wall_a = 476
spr_small_wall_b = 477
spr_small_wall_c = 478
spr_small_wall_c_destroy = 479
spr_small_wall_d = 480
spr_small_wall_e = 481
spr_small_wall_f = 482
spr_small_wall_f2 = 483
spr_town_house_a = 484
spr_town_house_b = 485
spr_town_house_c = 486
spr_town_house_d = 487
spr_town_house_e = 488
spr_town_house_f = 489
spr_town_house_g = 490
spr_town_house_h = 491
spr_town_house_i = 492
spr_town_house_j = 493
spr_town_house_l = 494
spr_town_house_m = 495
spr_town_house_n = 496
spr_town_house_o = 497
spr_town_house_p = 498
spr_town_house_q = 499
spr_passage_house_a = 500
spr_passage_house_b = 501
spr_passage_house_c = 502
spr_passage_house_d = 503
spr_passage_house_c_door = 504
spr_house_extension_a = 505
spr_house_extension_b = 506
spr_house_extension_c = 507
spr_house_extension_d = 508
spr_house_extension_e = 509
spr_house_extension_f = 510
spr_house_extension_f2 = 511
spr_house_extension_g = 512
spr_house_extension_g2 = 513
spr_house_extension_h = 514
spr_house_extension_i = 515
spr_house_roof_door = 516
spr_door_extension_a = 517
spr_stairs_arch_a = 518
spr_town_house_r = 519
spr_town_house_s = 520
spr_town_house_t = 521
spr_town_house_u = 522
spr_town_house_v = 523
spr_town_house_w = 524
spr_town_house_y = 525
spr_town_house_z = 526
spr_town_house_za = 527
spr_windmill = 528
spr_windmill_fan_turning = 529
spr_windmill_fan = 530
spr_fake_house_a = 531
spr_fake_house_b = 532
spr_fake_house_c = 533
spr_fake_house_d = 534
spr_fake_house_e = 535
spr_fake_house_f = 536
spr_fake_house_snowy_a = 537
spr_fake_house_snowy_b = 538
spr_fake_house_snowy_c = 539
spr_fake_house_snowy_d = 540
spr_fake_house_far_a = 541
spr_fake_house_far_b = 542
spr_fake_house_far_c = 543
spr_fake_house_far_d = 544
spr_fake_house_far_e = 545
spr_fake_house_far_f = 546
spr_fake_house_far_snowycrude_a = 547
spr_fake_house_far_snowy_b = 548
spr_fake_house_far_snowy_c = 549
spr_fake_house_far_snowy_d = 550
spr_earth_wall_a = 551
spr_earth_wall_a2 = 552
spr_earth_wall_b = 553
spr_earth_wall_b2 = 554
spr_earth_stairs_a = 555
spr_earth_stairs_b = 556
spr_earth_tower_small_a = 557
spr_earth_gate_house_a = 558
spr_earth_gate_a = 559
spr_earth_square_keep_a = 560
spr_earth_house_a = 561
spr_earth_house_b = 562
spr_earth_house_c = 563
spr_earth_house_d = 564
spr_village_steppe_a = 565
spr_village_steppe_b = 566
spr_village_steppe_c = 567
spr_village_steppe_d = 568
spr_village_steppe_e = 569
spr_village_steppe_f = 570
spr_town_house_aa = 571
spr_snowy_house_a = 572
spr_snowy_house_b = 573
spr_snowy_house_c = 574
spr_snowy_house_d = 575
spr_snowy_house_e = 576
spr_snowy_house_f = 577
spr_snowy_house_g = 578
spr_snowy_house_h = 579
spr_snowy_house_i = 580
spr_snowy_wall_a = 581
spr_snowy_stand = 582
spr_snowy_heap_a = 583
spr_snowy_trunks_a = 584
spr_snowy_castle_tower_a = 585
spr_snowy_castle_battlement_a = 586
spr_snowy_castle_battlement_a_destroyed = 587
spr_snowy_castle_battlement_b = 588
spr_snowy_castle_battlement_corner_a = 589
spr_snowy_castle_battlement_corner_b = 590
spr_snowy_castle_battlement_corner_c = 591
spr_snowy_castle_battlement_stairs_a = 592
spr_snowy_castle_battlement_stairs_b = 593
spr_snowy_castle_gate_house_a = 594
spr_snowy_castle_round_tower_a = 595
spr_snowy_castle_square_keep_a = 596
spr_snowy_castle_stairs_a = 597
spr_square_keep_b = 598
spr_square_keep_c = 599
spr_square_keep_d = 600
spr_square_keep_e = 601
spr_square_keep_f = 602
spr_square_extension_a = 603
spr_square_stairs_a = 604
spr_castle_courtyard_house_a = 605
spr_castle_courtyard_house_b = 606
spr_castle_courtyard_house_c = 607
spr_castle_courtyard_a = 608
spr_gatehouse_b = 609
spr_castle_gaillard = 610
spr_castle_e_battlement_a = 611
spr_castle_e_battlement_c = 612
spr_castle_e_battlement_a_destroyed = 613
spr_castle_e_sally_door_a = 614
spr_castle_e_corner = 615
spr_castle_e_corner_b = 616
spr_castle_e_corner_c = 617
spr_castle_e_stairs_a = 618
spr_castle_e_tower = 619
spr_castle_e_gate_house_a = 620
spr_castle_e_keep_a = 621
spr_stand_thatched = 622
spr_stand_cloth = 623
spr_castle_e_house_a = 624
spr_castle_e_house_b = 625
spr_arena_block_a = 626
spr_arena_block_b = 627
spr_arena_block_c = 628
spr_arena_block_d = 629
spr_arena_block_e = 630
spr_arena_block_f = 631
spr_arena_block_g = 632
spr_arena_block_h = 633
spr_arena_block_i = 634
spr_arena_block_j = 635
spr_arena_block_j_awning = 636
spr_arena_palisade_a = 637
spr_arena_wall_a = 638
spr_arena_wall_b = 639
spr_arena_barrier_a = 640
spr_arena_barrier_b = 641
spr_arena_barrier_c = 642
spr_arena_tower_a = 643
spr_arena_tower_b = 644
spr_arena_tower_c = 645
spr_arena_spectator_a = 646
spr_arena_spectator_b = 647
spr_arena_spectator_c = 648
spr_arena_spectator_sitting_a = 649
spr_arena_spectator_sitting_b = 650
spr_arena_spectator_sitting_c = 651
spr_courtyard_gate_a = 652
spr_courtyard_gate_b = 653
spr_courtyard_gate_c = 654
spr_courtyard_gate_snowy = 655
spr_castle_tower_a = 656
spr_castle_battlement_a = 657
spr_castle_battlement_b = 658
spr_castle_battlement_c = 659
spr_castle_battlement_a_destroyed = 660
spr_castle_battlement_b_destroyed = 661
spr_castle_battlement_corner_a = 662
spr_castle_battlement_corner_b = 663
spr_castle_battlement_corner_c = 664
spr_castle_battlement_stairs_a = 665
spr_castle_battlement_stairs_b = 666
spr_castle_gate_house_a = 667
spr_castle_round_tower_a = 668
spr_castle_square_keep_a = 669
spr_castle_stairs_a = 670
spr_castle_drawbridge_open = 671
spr_castle_drawbridge_closed = 672
spr_spike_group_a = 673
spr_spike_a = 674
spr_belfry_a = 675
spr_belfry_b = 676
spr_belfry_b_platform_a = 677
spr_belfry_old = 678
spr_belfry_platform_a = 679
spr_belfry_platform_b = 680
spr_belfry_platform_old = 681
spr_belfry_wheel = 682
spr_belfry_wheel_old = 683
spr_mangonel = 684
spr_trebuchet_old = 685
spr_trebuchet_new = 686
spr_trebuchet_destructible = 687
spr_stone_ball = 688
spr_village_house_a = 689
spr_village_house_b = 690
spr_village_house_c = 691
spr_village_house_d = 692
spr_farm_house_a = 693
spr_farm_house_b = 694
spr_farm_house_c = 695
spr_mountain_house_a = 696
spr_mountain_house_b = 697
spr_village_hut_a = 698
spr_crude_fence = 699
spr_crude_fence_small = 700
spr_crude_fence_small_b = 701
spr_ramp_12m = 702
spr_ramp_14m = 703
spr_siege_ladder_6m = 704
spr_siege_ladder_8m = 705
spr_siege_ladder_10m = 706
spr_siege_ladder_12m = 707
spr_siege_ladder_14m = 708
spr_siege_ladder_move_6m = 709
spr_siege_ladder_move_8m = 710
spr_siege_ladder_move_10m = 711
spr_siege_ladder_move_12m = 712
spr_siege_ladder_move_14m = 713
spr_portcullis = 714
spr_bed_a = 715
spr_bed_b = 716
spr_bed_c = 717
spr_bed_d = 718
spr_bed_e = 719
spr_bed_f = 720
spr_towngate_door_left = 721
spr_towngate_door_right = 722
spr_towngate_rectangle_door_left = 723
spr_towngate_rectangle_door_right = 724
spr_door_screen = 725
spr_door_a = 726
spr_door_b = 727
spr_door_c = 728
spr_door_d = 729
spr_tavern_door_a = 730
spr_tavern_door_b = 731
spr_door_e_left = 732
spr_door_e_right = 733
spr_door_f_left = 734
spr_door_f_right = 735
spr_door_h_left = 736
spr_door_h_right = 737
spr_draw_bridge_a = 738
spr_chain_1m = 739
spr_chain_2m = 740
spr_chain_5m = 741
spr_chain_10m = 742
spr_bridge_modular_a = 743
spr_bridge_modular_b = 744
spr_church_a = 745
spr_church_tower_a = 746
spr_stone_step_a = 747
spr_stone_step_b = 748
spr_stone_step_c = 749
spr_stone_heap = 750
spr_stone_heap_b = 751
spr_panel_door_a = 752
spr_panel_door_b = 753
spr_smoke_stain = 754
spr_brazier_with_fire = 755
spr_cooking_fire = 756
spr_cauldron_a = 757
spr_fry_pan_a = 758
spr_tripod_cauldron_a = 759
spr_tripod_cauldron_b = 760
spr_open_stable_a = 761
spr_open_stable_b = 762
spr_plate_a = 763
spr_plate_b = 764
spr_plate_c = 765
spr_lettuce = 766
spr_hanger = 767
spr_knife_eating = 768
spr_colander = 769
spr_ladle = 770
spr_spoon = 771
spr_skewer = 772
spr_grape_a = 773
spr_grape_b = 774
spr_apple_a = 775
spr_apple_b = 776
spr_maize_a = 777
spr_maize_b = 778
spr_cabbage = 779
spr_flax_bundle = 780
spr_olive_plane = 781
spr_grapes_plane = 782
spr_date_fruit_plane = 783
spr_bowl = 784
spr_bowl_small = 785
spr_dye_blue = 786
spr_dye_red = 787
spr_dye_yellow = 788
spr_basket = 789
spr_basket_big = 790
spr_basket_big_green = 791
spr_leatherwork_frame = 792
spr_cabbage_b = 793
spr_bean = 794
spr_basket_a = 795
spr_feeding_trough_a = 796
spr_marrow_a = 797
spr_marrow_b = 798
spr_squash_plant = 799
spr_gatehouse_new_a = 800
spr_gatehouse_new_b = 801
spr_gatehouse_new_snowy_a = 802
spr_winch = 803
spr_winch_b = 804
spr_drawbridge = 805
spr_gatehouse_door_left = 806
spr_gatehouse_door_right = 807
spr_cheese_a = 808
spr_cheese_b = 809
spr_cheese_slice_a = 810
spr_bread_a = 811
spr_bread_b = 812
spr_bread_slice_a = 813
spr_fish_a = 814
spr_fish_roasted_a = 815
spr_chicken_roasted = 816
spr_food_steam = 817
spr_city_smoke = 818
spr_city_fire_fly_night = 819
spr_city_fly_day = 820
spr_flue_smoke_tall = 821
spr_flue_smoke_short = 822
spr_moon_beam = 823
spr_fire_small = 824
spr_fire_big = 825
spr_battle_field_smoke = 826
spr_Village_fire_big = 827
spr_candle_a = 828
spr_candle_b = 829
spr_candle_c = 830
spr_lamp_a = 831
spr_lamp_b = 832
spr_hook_a = 833
spr_window_night = 834
spr_fried_pig = 835
spr_village_oven = 836
spr_dungeon_water_drops = 837
spr_shadow_circle_1 = 838
spr_shadow_circle_2 = 839
spr_shadow_square_1 = 840
spr_shadow_square_2 = 841
spr_wheelbarrow = 842
spr_gourd = 843
spr_gourd_spike = 844
spr_obstacle_fence_1 = 845
spr_obstacle_fallen_tree_a = 846
spr_obstacle_fallen_tree_b = 847
spr_siege_wall_a = 848
spr_siege_large_shield_a = 849
spr_granary_a = 850
spr_small_wall_connect_a = 851
spr_full_stable_a = 852
spr_full_stable_b = 853
spr_full_stable_c = 854
spr_full_stable_d = 855
spr_arabian_house_a = 856
spr_arabian_house_b = 857
spr_arabian_house_c = 858
spr_arabian_house_d = 859
spr_arabian_house_e = 860
spr_arabian_house_f = 861
spr_arabian_house_g = 862
spr_arabian_house_h = 863
spr_arabian_house_i = 864
spr_arabian_square_keep_a = 865
spr_arabian_passage_house_a = 866
spr_arabian_wall_a = 867
spr_arabian_wall_b = 868
spr_arabian_ground_a = 869
spr_arabian_parterre_a = 870
spr_well_shaft = 871
spr_horse_mill = 872
spr_horse_mill_collar = 873
spr_arabian_stable = 874
spr_arabian_tent = 875
spr_arabian_tent_b = 876
spr_desert_plant_a = 877
spr_arabian_castle_battlement_a = 878
spr_arabian_castle_battlement_b_destroyed = 879
spr_arabian_castle_battlement_c = 880
spr_arabian_castle_battlement_d = 881
spr_arabian_castle_corner_a = 882
spr_arabian_castle_stairs = 883
spr_arabian_castle_stairs_b = 884
spr_arabian_castle_stairs_c = 885
spr_arabian_castle_battlement_section_a = 886
spr_arabian_castle_gate_house_a = 887
spr_arabian_castle_house_a = 888
spr_arabian_castle_house_b = 889
spr_arabian_castle_keep_a = 890
spr_arabian_house_a2 = 891
spr_arabian_village_house_a = 892
spr_arabian_village_house_b = 893
spr_arabian_village_house_c = 894
spr_arabian_village_house_d = 895
spr_arabian_village_stable = 896
spr_arabian_village_hut = 897
spr_arabian_village_stairs = 898
spr_tree_a01 = 899
spr_stairs_a = 900
spr_headquarters_flag_red = 901
spr_headquarters_flag_blue = 902
spr_headquarters_flag_gray = 903
spr_headquarters_flag_red_code_only = 904
spr_headquarters_flag_blue_code_only = 905
spr_headquarters_flag_gray_code_only = 906
spr_headquarters_pole_code_only = 907
spr_headquarters_flag_swadian = 908
spr_headquarters_flag_vaegir = 909
spr_headquarters_flag_khergit = 910
spr_headquarters_flag_nord = 911
spr_headquarters_flag_rhodok = 912
spr_headquarters_flag_sarranid = 913
spr_glow_a = 914
spr_glow_b = 915
spr_arabian_castle_corner_b = 916
spr_dummy_a_undestructable = 917
spr_cave_entrance_1 = 918
spr_pointer_arrow = 919
spr_fireplace_d_interior = 920
spr_ship_sail_off = 921
spr_ship_sail_off_b = 922
spr_ship_c_sail_off = 923
spr_ramp_small_a = 924
spr_castle_g_battlement_b = 925
spr_box_a_dynamic = 926
spr_desert_field = 927
spr_water_river = 928
spr_viking_house_a = 929
spr_viking_house_b = 930
spr_viking_house_c = 931
spr_viking_house_d = 932
spr_viking_house_e = 933
spr_viking_stable_a = 934
spr_viking_keep = 935
spr_viking_house_c_destroy = 936
spr_viking_house_b_destroy = 937
spr_harbour_a = 938
spr_sea_foam_a = 939
spr_viking_keep_destroy = 940
spr_viking_keep_destroy_door = 941
spr_earth_tower_small_b = 942
spr_earth_gate_house_b = 943
spr_earth_tower_a = 944
spr_earth_stairs_c = 945
spr_earth_sally_gate_left = 946
spr_earth_sally_gate_right = 947
spr_barrier_box = 948
spr_barrier_capsule = 949
spr_barrier_cone = 950
spr_barrier_sphere = 951
spr_viking_keep_destroy_sally_door_right = 952
spr_viking_keep_destroy_sally_door_left = 953
spr_castle_f_door_b = 954
spr_ctf_flag_kingdom_1 = 955
spr_ctf_flag_kingdom_2 = 956
spr_ctf_flag_kingdom_3 = 957
spr_ctf_flag_kingdom_4 = 958
spr_ctf_flag_kingdom_5 = 959
spr_ctf_flag_kingdom_6 = 960
spr_ctf_flag_kingdom_7 = 961
spr_headquarters_flag_rebel = 962
spr_arabian_lighthouse_a = 963
spr_arabian_ramp_a = 964
spr_arabian_ramp_b = 965
spr_winery_interior = 966
spr_winery_barrel_shelf = 967
spr_winery_wall_shelf = 968
spr_winery_huge_barrel = 969
spr_winery_wine_press = 970
spr_winery_middle_barrel = 971
spr_winery_wine_cart_small_loaded = 972
spr_winery_wine_cart_small_empty = 973
spr_winery_wine_cart_empty = 974
spr_winery_wine_cart_loaded = 975
spr_weavery_interior = 976
spr_weavery_loom_a = 977
spr_weavery_spinning_wheel = 978
spr_mill_interior = 979
spr_mill_flour_sack = 980
spr_mill_flour_sack_desk_a = 981
spr_mill_flour_sack_desk_b = 982
spr_smithy_interior = 983
spr_smithy_grindstone_wheel = 984
spr_smithy_forge_bellows = 985
spr_smithy_forge = 986
spr_smithy_anvil = 987
spr_tannery_hide_a = 988
spr_tannery_hide_b = 989
spr_tannery_pools_a = 990
spr_tannery_pools_b = 991
spr_fountain = 992
spr_rhodok_houses_a = 993
spr_rhodok_houses_b = 994
spr_rhodok_houses_c = 995
spr_rhodok_houses_d = 996
spr_rhodok_houses_e = 997
spr_rhodok_house_passage_a = 998
spr_bridge_b = 999
spr_brewery_pool = 1000
spr_brewery_big_bucket = 1001
spr_brewery_interior = 1002
spr_brewery_bucket_platform_a = 1003
spr_brewery_bucket_platform_b = 1004
spr_weavery_dye_pool_r = 1005
spr_weavery_dye_pool_y = 1006
spr_weavery_dye_pool_b = 1007
spr_weavery_dye_pool_p = 1008
spr_weavery_dye_pool_g = 1009
spr_oil_press_interior = 1010
spr_city_swad_01 = 1011
spr_city_swad_02 = 1012
spr_city_swad_03 = 1013
spr_city_swad_04 = 1014
spr_city_swad_passage_01 = 1015
spr_city_swad_05 = 1016
spr_arena_block_j_a = 1017
spr_arena_underway_a = 1018
spr_arena_circle_a = 1019
spr_rope_bridge_15m = 1020
spr_tree_house_a = 1021
spr_tree_house_guard_a = 1022
spr_tree_house_guard_b = 1023
spr_tree_shelter_a = 1024
spr_yellow_fall_leafs_a = 1025
spr_rock_bridge_a = 1026
spr_suspension_bridge_a = 1027
spr_mine_a = 1028



from smart_module.module_system.header_common import *
from smart_module.module_system.header_parties import *
from smart_module.module_system.ID_troops import *
from smart_module.module_system.ID_factions import *
from smart_module.module_system.ID_party_templates import *
from smart_module.module_system.ID_map_icons import *

####################################################################################################################
#  Each party record contains the following fields:
#  1) Party id: used for referencing parties in other files.
#     The prefix p_ is automatically added before each party id.
#  2) Party name.
#  3) Party flags. See header_parties.py for a list of available flags
#  4) Menu. ID of the menu to use when this party is met. The value 0 uses the default party encounter system.
#  5) Party-template. ID of the party template this party belongs to. Use pt_none as the default value.
#  6) Faction.
#  7) Personality. See header_parties.py for an explanation of personality flags.
#  8) Ai-behavior
#  9) Ai-target party
# 10) Initial coordinates.
# 11) List of stacks. Each stack record is a triple that contains the following fields:
#   11.1) Troop-id. 
#   11.2) Number of troops in this stack. 
#   11.3) Member flags. Use pmf_is_prisoner to note that this member is a prisoner.
# 12) Party direction in degrees [optional]
####################################################################################################################

no_menu = 0
#pf_town = pf_is_static|pf_always_visible|pf_hide_defenders|pf_show_faction
pf_town = pf_is_static|pf_always_visible|pf_show_faction|pf_label_large
pf_castle = pf_is_static|pf_always_visible|pf_show_faction|pf_label_medium
pf_village = pf_is_static|pf_always_visible|pf_hide_defenders|pf_label_small

#sample_party = [(trp_swadian_knight,1,0), (trp_swadian_peasant,10,0), (trp_swadian_crossbowman,1,0), (trp_swadian_man_at_arms, 1, 0), (trp_swadian_footman, 1, 0), (trp_swadian_militia,1,0)]

# NEW TOWNS:
# NORMANDY: Rouen, Caen, Bayeux, Coutances, Evreux, Avranches
# Brittany: Rennes, Nantes,
# Maine: Le Mans
# Anjou: Angers


parties = [
  ("main_party","Main Party",icon_player|pf_limit_members, no_menu, pt_none,fac_player_faction,0,ai_bhvr_hold,0,(17, 52.5),[(trp_player,1,0)]),
  ("temp_party","{!}temp_party",pf_disabled, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0,0),[]),
  ("camp_bandits","{!}camp_bandits",pf_disabled, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(1,1),[(trp_temp_troop,3,0)]),
#parties before this point are hardwired. Their order should not be changed.

  ("temp_party_2","{!}temp_party_2",pf_disabled, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0,0),[]),
#Used for calculating casulties.
  ("temp_casualties","{!}casualties",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("temp_casualties_2","{!}casualties",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("temp_casualties_3","{!}casualties",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("temp_wounded","{!}enemies_wounded",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("temp_killed", "{!}enemies_killed", pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("main_party_backup","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("encountered_party_backup","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
#  ("ally_party_backup","_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("collective_friends_backup","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("player_casualties","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("enemy_casualties","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("ally_casualties","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),

  ("collective_enemy","{!}collective_enemy",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  #TODO: remove this and move all to collective ally
  ("collective_ally","{!}collective_ally",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
  ("collective_friends","{!}collective_ally",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),
   
  ("total_enemy_casualties","{!}_",  pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]), #ganimet hesaplari icin #new:
  ("routed_enemies","{!}routed_enemies",pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]), #new:  

#  ("village_reinforcements","village_reinforcements",pf_is_static|pf_disabled, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(1,1),[]),

###############################################################  
  ("zendar","Zendar",pf_disabled|icon_town|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(18,60),[]),

  ("town_1","Sargoth",  icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-17.6, 79.7),[], 170),
  ("town_2","Tihr",     icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-53.5, 78.4),[], 120),
  ("town_3","Veluca",   icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-57.4, -44.5),[], 80),
  ("town_4","Suno",     icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-70, 15.4),[], 290),
  ("town_5","Jelkala",  icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-74.6, -79.7),[], 90),
  ("town_6","Praven",   icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-96, 26.4),[], 155),
  ("town_7","Uxkhal",   icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-50, -8.5),[], 240),

  ("town_8","Reyvadin", icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(48.44, 39.3),[], 175),
  ("town_9","Khudan",   icon_town_snow|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(94, 65.2),[], 90),
  ("town_10","Tulga",   icon_town_steppe|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(135.5, -22),[], 310),
  ("town_11","Curaw",   icon_town_snow|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(43, 67.5),[], 150),
  ("town_12","Wercheg", icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-1.2, 108.9),[], 25),
  ("town_13","Rivacheg",icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(64.8, 113.7),[], 60),
  ("town_14","Halmar",  icon_town_steppe|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(55.5, -45),[], 135),

  ("town_15","Yalen",  icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-132.8, -47.3),[], 45),
  ("town_16","Dhirim",  icon_town|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(14, -2),[], 0),
  ("town_17","Ichamur",  icon_town_steppe|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(121.8, 8.6),[], 90),
  ("town_18","Narra",  icon_town_steppe|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(88, -26.5),[], 135),

  ("town_19","Shariz", icon_town_desert|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(15, -107),[], 45),
  ("town_20","Durquba", icon_town_desert|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(90, -95.1),[], 270),
  ("town_21","Ahmerrad", icon_town_desert|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(130.5, -78.5),[], 330),
  ("town_22","Bariyye", icon_town_desert|pf_town, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(165, -106.7),[], 225),

#   Aztaq_Castle       
#  Malabadi_Castle
  ("castle_1","Culmarr_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-101.3, -21),[],50),
  ("castle_2","Malayurg_Castle",icon_castle_b|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(97.5, -2.2),[],75),
  ("castle_3","Bulugha_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(47.5, 111.3),[],100),
  ("castle_4","Radoghir_Castle",icon_castle_c|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(32.5, 47.8),[],180),
  ("castle_5","Tehlrog_Castle",icon_castle_c|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-4.8, 63.7),[],90),
  ("castle_6","Tilbaut_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(37.6, 17.1),[],55),
  ("castle_7","Sungetche_Castle",icon_castle_snow_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(109.5, 41.5),[],45),
  ("castle_8","Jeirbe_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(35.2, 89),[],30),
  ("castle_9","Jamiche_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-7.5, -82.6),[],100),
  ("castle_10","Alburq_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(24.2, 96.85),[],110),
  ("castle_11","Curin_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-27.50, 83.46),[],75),
  ("castle_12","Chalbek_Castle",icon_castle_b|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-84.75, 105.5),[],95),
  ("castle_13","Kelredan_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-10.6, 17.6),[],115),
  ("castle_14","Maras_Castle",icon_castle_c|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-122.4, -18.1),[],90),
  ("castle_15","Ergellon_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-52.5, -28),[],235),
  ("castle_16","Almerra_Castle",icon_castle_c|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-10.6, -65.6),[],45),
  ("castle_17","Distar_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(140.3, -10.8),[],15),
  ("castle_18","Ismirala_Castle",icon_castle_snow_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(14.4, 70.1),[],300),
  ("castle_19","Yruma_Castle",icon_castle_snow_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(69.5, 55.6),[],280),
  ("castle_20","Derchios_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(16, 11.5),[],260),
  ("castle_21","Ibdeles_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-73, -89.5),[],260),
  ("castle_22","Unuzdaq_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(33, -64),[],260),
  ("castle_23","Tevarin_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-124.8, 44.3),[],80),
  ("castle_24","Reindi_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(24.9, -35.6),[],260),
  ("castle_25","Ryibelet_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-57, 30.6),[],260),
  ("castle_26","Senuzgda_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-2.5, -9.5),[],260),
  ("castle_27","Rindyar_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(63.3, -2),[],260),
  ("castle_28","Grunwalder_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-36.4, -39.3),[],260),

  ("castle_29","Nelag_Castle",icon_castle_snow_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(147.7, 50.4),[],280),
  ("castle_30","Asugan_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(176, -47),[],260),
  ("castle_31","Vyincourd_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-65.7, -12.5),[],260),
  ("castle_32","Knudarr_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(2, 30.1),[],260),
  ("castle_33","Etrosq_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-101.4, -32.1),[],80),
  ("castle_34","Hrus_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-72.5, 78.6),[],260),
  ("castle_35","Haringoth_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-110, 0),[],260),
  ("castle_36","Jelbegi_Castle",icon_castle_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-47.3, 53.2),[],260),
  ("castle_37","Dramug_Castle",icon_castle_c|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(55.3, 23),[],260),
  ("castle_38","Tulbuk_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(141.7, 33.3),[],260),
  ("castle_39","Slezkh_Castle",icon_castle_snow_a|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(56.8, 74.5),[],280),
  ("castle_40","Uhhun_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(62.9, -26),[],260),

  ("castle_41","Jameyyed_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(71.3, -71.1),[],260),
  ("castle_42","Teramma_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(70, -96),[],80),
  ("castle_43","Sharwa_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(172, -65),[],260),
  ("castle_44","Durrin_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(128, -87),[],260),
  ("castle_45","Caraf_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(30.6, -110.6),[],260),
  ("castle_46","Weyyah_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(13.3, -84.4),[],260),
  ("castle_47","Samarra_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(116, -74),[],260),
  ("castle_48","Bardaq_Castle",icon_castle_d|pf_castle, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(157, -80),[],260),
 

#     Rinimad      
#              Rietal Derchios Gerdus
# Tuavus   Pamir   vezona 
  
  ("village_1", "Yaragar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-60, -9.5),[], 100),
  ("village_2", "Burglen",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-13.5, 3.5),[], 110),
  ("village_3", "Azgad",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-97.4, 36),[], 120),
  ("village_4", "Nomar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-36.6, -13.2),[], 130),
  ("village_5", "Kulum",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-122.7, 106.3),[], 170),
  ("village_6", "Emirin",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(5.5, -2.5),[], 100),
  ("village_7", "Amere",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(39.3, -5.25),[], 110),
  ("village_8", "Haen",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-49.7, 74),[], 120),
  ("village_9", "Buvran",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-85, -75.35),[], 130),
  ("village_10","Mechin",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(8.8, 34.75),[], 170),

  ("village_11","Dusturil",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(137.2, -36.5),[], 100),
  ("village_12","Emer",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-45.8, -58.5),[], 110),
  ("village_13","Nemeja",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-119, 3),[], 120),
  ("village_14","Sumbuja",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(40,52),[], 130),
  ("village_15","Ryibelet",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-49.3, 26.25),[], 170),
  ("village_16","Shapeshte",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(74, 86.8),[], 170),
  ("village_17","Mazen",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(44.7, 91.4),[], 35),
  ("village_18","Ulburban",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(73.7, 30.1),[], 170),
  ("village_19","Hanun",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(152.3, 53.5),[], 170),
  ("village_20","Uslum",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(116, 80.3),[], 170),

  ("village_21","Bazeck",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(60.4, 85),[], 100),
  ("village_22","Shulus",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(125.4, 64),[], 110),
  ("village_23","Ilvia",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-133.3, -37.6),[], 120),
  ("village_24","Ruldi",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-57.6, -73),[], 130),
  ("village_25","Dashbigha",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(125.2, -8),[], 170),
  ("village_26","Pagundur",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-58.5, -30.8),[], 170),
  ("village_27","Glunmar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-146.3, -16.6),[], 170),
  ("village_28","Tash_Kulun",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(95, -11.4),[], 170),
  ("village_29","Buillin",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-90.6, 110.9),[], 170),

  ("village_30","Ruvar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(29.2, 114.3),[], 170),
  ("village_31","Ambean",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-24, 53),[], 100),
  ("village_32","Tosdhar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(8.3 , 14.5 ),[], 110),
  ("village_33","Ruluns",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-59, 10.6),[], 120),
  ("village_34","Ehlerdah",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(34.15, -30),[], 130),
  ("village_35","Fearichen",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(2.14, 86.9),[], 170),
  ("village_36","Jayek",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(17.4, 100.7),[], 170),
  ("village_37","Ada_Kulun",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(164.4, 26),[], 170),
  ("village_38","Ibiran",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-42.8, 11.25),[], 170),
  ("village_39","Reveran",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-124.3, -29.5),[], 170),
  ("village_40","Saren",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-12, -53),[], 170),

  ("village_41","Dugan",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(175, -39.5),[], 100),
  ("village_42","Dirigh_Aban",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(115.4, 21.6),[], 110),
  ("village_43","Zagush",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(99.4, -21.3),[], 120),
  ("village_44","Peshmi",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(51.8, -50),[], 130),
  ("village_45","Bulugur",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(151.5, -3),[], 170),
  ("village_46","Fedner",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-82.4, -48.8),[], 170),
  ("village_47","Epeshe",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-101.2, -53.7),[], 170),
  ("village_48","Veidar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-103, 15.3),[], 170),
  ("village_49","Tismirr",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(90.8, 60.5),[], 10),
  ("village_50","Karindi",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(64.1, 55.9),[], 170),

  ("village_51","Jelbegi",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-40, 62),[], 100),
  ("village_52","Amashke",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(38.2, -67.7),[], 110),
  ("village_53","Balanli",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-130.3, 42),[], 120),
  ("village_54","Chide",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-19, 15.5),[], 130),
  ("village_55","Tadsamesh",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(37.3, 23.7),[], 170),
  ("village_56","Fenada",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-5.55, 75.5),[], 170),
  ("village_57","Ushkuru",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(24.75, -7.7),[], 170),
  ("village_58","Vezin",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(78.50, 118.3),[], 170),
  ("village_59","Dumar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-101, -45),[], 170),
  ("village_60","Tahlberl",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-26.6, 30),[], 170),

  ("village_61","Aldelen",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-99.8, 85.65),[], 100),
  ("village_62","Rebache",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(36.7, 59.5),[], 100),
  ("village_63","Rduna",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(70.75, 1.6),[], 100),
  ("village_64","Serindiar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-41.5, -35.8),[], 100),
  ("village_65","Iyindah",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-86.3, 12.4),[], 100),
  ("village_66","Fisdnar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(109, 127.5),[], 100),
  ("village_67","Tebandra",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(65, 23),[], 100),
  ("village_68","Ibdeles",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-72.6, -97.5),[], 100),
  ("village_69","Kwynn",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-26.5, 77.6),[], 100),
  ("village_70","Dirigsene",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-94, -27),[], 100),

  ("village_71","Tshibtin",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-9, -20),[], 20),
  ("village_72","Elberl",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-144, 16.15),[], 60),
  ("village_73","Chaeza",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-63.2, -51.4),[], 55),
  ("village_74","Ayyike",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(49, 31),[], 15),
  ("village_75","Bhulaban",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(110.2, 48.8),[], 10),
  ("village_76","Kedelke",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(91.5, -34.8),[], 35),
  ("village_77","Rizi",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-64.8, 81.5),[], 160),
  ("village_78","Sarimish",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-30.2, -53.3),[], 180),
  ("village_79","Istiniar",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-134.1, -5.5),[], 0),
  ("village_80","Vayejeg",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-1.5, 56),[], 40),

  ("village_81","Odasan",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-17.2, 123.6),[], 20),
  ("village_82","Yalibe",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(12, -26),[], 60),
  ("village_83","Gisim",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-79.9, 55.2),[], 55),
  ("village_84","Chelez",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-45.6, -83),[], 15),
  ("village_85","Ismirala",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(22.8, 71.7),[], 10),
  ("village_86","Slezkh",  icon_village_snow_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(60.5, 68.2),[], 35),
  ("village_87","Udiniad",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(52.3, 111.8),[], 160),
  ("village_88","Tulbuk",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(135.8, 24.7),[], 180),
  ("village_89","Uhhun",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(70.1, -27.9),[], 0),
  ("village_90","Jamiche",  icon_village_a|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-12.7, -85.5),[], 40),

  ("village_91","Ayn Assuadi",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(21.6, -95.3),[], 20),
  ("village_92","Dhibbain",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(42.5, -111.6),[], 60),
  ("village_93","Qalyut",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(50, -94.8),[], 55),
  ("village_94","Mazigh",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(75.4, -61.65),[], 15),
  ("village_95","Tamnuh",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(99, -90.5),[], 10),
  ("village_96","Habba",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(107, -75),[], 35),
  ("village_97","Sekhtem",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(82.3, -76.8),[], 160),
  ("village_98","Mawiti",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(117.7, -62.2),[], 180),
  ("village_99","Fishara",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(158.35, -98),[], 0),
  ("village_100","Iqbayl",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(150, -106.5),[], 40),

  ("village_101","Uzgha",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(154, -69.5),[], 20),
  ("village_102","Shibal Zumr",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(77.2, -91),[], 60),
  ("village_103","Mijayet",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(121, -95.6),[], 55),
  ("village_104","Tazjunat",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(159, -58.5),[], 15),
  ("village_105","Aab",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(135, -88.5),[], 10),
  ("village_106","Hawaha",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(10.3, -91.0),[], 35),
  ("village_107","Unriya",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(156.6, -84),[], 160),
  ("village_108","Mit Nun",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(28.8, -107.3),[], 180),
  ("village_109","Tilimsal",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(53, -114.5),[], 0),
  ("village_110","Rushdigh",  icon_village_c|pf_village, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(38, -104),[], 40),
  
  ("salt_mine","Salt_Mine",icon_village_a|pf_disabled|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(14.2, -31),[]),
  ("four_ways_inn","Four_Ways_Inn",icon_village_a|pf_disabled|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(4.8, -39.6),[]),
  ("test_scene","test_scene",icon_village_a|pf_disabled|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(10.8, -19.6),[]),
  #("test_scene","test_scene",icon_village_a|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(10.8, -19.6),[]),
  ("battlefields","battlefields",pf_disabled|icon_village_a|pf_is_static|pf_always_visible|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(10.8, -16.6),[]),
  ("dhorak_keep","Dhorak_Keep",icon_town|pf_disabled|pf_is_static|pf_always_visible|pf_no_label|pf_hide_defenders, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-50,-58),[]),

  ("training_ground","Training Ground",  pf_disabled|icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(3, -7),[]),

  ("training_ground_1", "Training Field",  icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible|pf_label_medium, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(37.4, 102.8),[], 100),
  ("training_ground_2", "Training Field",  icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible|pf_label_medium, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-12.8, 33),[], 100),
  ("training_ground_3", "Training Field",  icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible|pf_label_medium, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(70.5, 72.0),[], 100),
  ("training_ground_4", "Training Field",  icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible|pf_label_medium, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(11.5, -75.2),[], 100),
  ("training_ground_5", "Training Field",  icon_training_ground|pf_hide_defenders|pf_is_static|pf_always_visible|pf_label_medium, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-125.8, 12.5),[], 100),


#  bridge_a
  ("Bridge_1","{!}1",icon_bridge_snow_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(39.37, 65.10),[], -44.8),
  ("Bridge_2","{!}2",icon_bridge_snow_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(56.44, 77.88),[], 4.28),
  ("Bridge_3","{!}3",icon_bridge_snow_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(70.87, 87.95),[], 64.5),
  ("Bridge_4","{!}4",icon_bridge_snow_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(93.71, 62.13),[], -2.13),
  ("Bridge_5","{!}5",icon_bridge_snow_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(11.02, 72.61),[], 21.5),
  ("Bridge_6","{!}6",icon_bridge_b|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-8.83, 52.24),[], -73.5),
  ("Bridge_7","{!}7",icon_bridge_b|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-29.79, 76.84),[], -64),
  ("Bridge_8","{!}8",icon_bridge_b|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-64.05, -6),[], 1.72),
  ("Bridge_9","{!}9",icon_bridge_b|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-64.95, -9.60),[], -33.76),
  ("Bridge_10","{!}10",icon_bridge_b|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-75.32, -75.27),[], -44.07),
  ("Bridge_11","{!}11",icon_bridge_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-24.39, 67.82),[], 81.3),
  ("Bridge_12","{!}12",icon_bridge_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-114.33, -1.94),[], -35.5),
  ("Bridge_13","{!}13",icon_bridge_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-84.02, -7),[], -17.7),
  ("Bridge_14","{!}14",icon_bridge_a|pf_is_static|pf_always_visible|pf_no_label, no_menu, pt_none, fac_neutral,0,ai_bhvr_hold,0,(-23.36, 75.8),[], 66.6),

  ("looter_spawn_point"   ,"{!}looter_sp",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(26, 77),[(trp_looter,15,0)]),
  ("steppe_bandit_spawn_point"  ,"the steppes",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(125, 9),[(trp_looter,15,0)]),
  ("taiga_bandit_spawn_point"   ,"the tundra",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(78, 84),[(trp_looter,15,0)]),
##  ("black_khergit_spawn_point"  ,"black_khergit_sp",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(47.1, -73.3),[(trp_looter,15,0)]),
  ("forest_bandit_spawn_point"  ,"the forests",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(-35, 18),[(trp_looter,15,0)]),
  ("mountain_bandit_spawn_point","the highlands",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(-90, -26.8),[(trp_looter,15,0)]),
  ("sea_raider_spawn_point_1"   ,"the coast",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(48.5, 110),[(trp_looter,15,0)]),
  ("sea_raider_spawn_point_2"   ,"the coast",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(-42, 76.7),[(trp_looter,15,0)]),
  ("desert_bandit_spawn_point"  ,"the deserts",pf_disabled|pf_is_static, no_menu, pt_none, fac_outlaws,0,ai_bhvr_hold,0,(110, -100),[(trp_looter,15,0)]),
 # add extra towns before this point 
  ("spawn_points_end"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ("reserved_1"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ("reserved_2"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ("reserved_3"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ("reserved_4"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ("reserved_5"                  ,"{!}last_spawn_point",    pf_disabled|pf_is_static, no_menu, pt_none, fac_commoners,0,ai_bhvr_hold,0,(0., 0),[(trp_looter,15,0)]),
  ]



from smart_module.module_system.module_animations import animations
from smart_module.module_system.module_dialogs import dialogs
from smart_module.module_system.module_factions import factions
from smart_module.module_system.module_game_menus import game_menus
from smart_module.module_system.module_info_pages import info_pages
from smart_module.module_system.module_items import items
from smart_module.module_system.module_map_icons import map_icons
from smart_module.module_system.module_meshes import meshes
from smart_module.module_system.module_mission_templates import mission_templates
from smart_module.module_system.module_music import tracks
from smart_module.module_system.module_particle_systems import particle_systems
from smart_module.module_system.module_parties import parties
from smart_module.module_system.module_party_templates import party_templates
from smart_module.module_system.module_postfx import postfx_params
from smart_module.module_system.module_presentations import presentations
from smart_module.module_system.module_quests import quests
from smart_module.module_system.module_scene_props import scene_props
from smart_module.module_system.module_scenes import scenes
from smart_module.module_system.module_scripts import scripts
from smart_module.module_system.module_simple_triggers import simple_triggers
from smart_module.module_system.module_skills import skills
from smart_module.module_system.module_skins import skins
from smart_module.module_system.module_sounds import sounds
from smart_module.module_system.module_strings import strings
from smart_module.module_system.module_tableau_materials import tableaus
from smart_module.module_system.module_triggers import triggers
from smart_module.module_system.module_troops import troops

################################################
###
### 所有数据映射集合
### datas:数据集合
### level:数据优先级别（数值越小级别越高）
###
################################################

moduleDataMaping = {
    "factions":{
        "datas":factions,
        "level":0,
    },
    "troops":{
        "datas":troops,
        "level":0,
    },
    "skills":{
        "datas":skills,
        "level":0,
    },
    "skins":{
        "datas":skins,
        "level":0,
    },
    "items":{
        "datas":items,
        "level":0,
    },
    "parties":{
        "datas":parties,
        "level":0,
    },
    "party_templates":{
        "datas":party_templates,
        "level":2,
    },
    "scripts":{
        "datas":scripts,
        "level":5,
    },
    "triggers":{
        "datas":triggers,
        "level":10,
    },
    "simple_triggers":{
        "datas":simple_triggers,
        "level":10,
    },
    "dialogs":{
        "datas":dialogs,
        "level":10,
    },
    "game_menus":{
        "datas":game_menus,
        "level":10,
    },
    "mission_templates":{
        "datas":mission_templates,
        "level":2,
    },
    "quests":{
        "datas":quests,
        "level":0,
    },
    "strings":{
        "datas":strings,
        "level":0,
    },
    "presentations":{
        "datas":presentations,
        "level":10,
    },
    "map_icons":{
        "datas":map_icons,
        "level":0,
    },
    "particle_systems":{
        "datas":particle_systems,
        "level":0,
    },
    "scenes":{
        "datas":scenes,
        "level":0,
    },
    "info_pages":{
        "datas":info_pages,
        "level":0,
    },
    "tableaus":{
        "datas":tableaus,
        "level":5,
    },
    "meshes":{
        "datas":meshes,
        "level":0,
    },
    "postfx_params":{
        "datas":postfx_params,
        "level":0,
    },
    "scene_props":{
        "datas":scene_props,
        "level":5,
    },
    "tracks":{
        "datas":tracks,
        "level":0,
    },
    "sounds":{
        "datas":sounds,
        "level":0,
    },
    "animations":{
        "datas":animations,
        "level":0,
    },
}



from smart_module.module_system.module_mission_templates import *

from smart_module.module_system.process_operations import *
from smart_module.module_system.process_tags_unused import tag_uses

mission_template_name_pos = 0
mission_template_flags_pos = 1
mission_template_types_pos = 2
mission_template_desc_pos = 3
mission_template_groups_pos =4
mission_template_triggers_pos = 5

def save_triggers(file,template_name,triggers,variable_list,variable_uses,tag_uses,quick_strings):
  file.write("%d\n"%len(triggers))
  for i in range(len(triggers)):
    trigger = triggers[i]
    file.write("%f %f %f "%(trigger[trigger_check_pos],trigger[trigger_delay_pos],trigger[trigger_rearm_pos]))
    save_statement_block(file, 0, 1, trigger[trigger_conditions_pos]  , variable_list,variable_uses,tag_uses,quick_strings)
    save_statement_block(file, 0, 1, trigger[trigger_consequences_pos], variable_list,variable_uses,tag_uses,quick_strings)
    file.write("\n")
  file.write("\n")

def save_mission_template_group(file,entry):
  if (len(entry[5]) > 8):
    ## print( "ERROR: Too many item_overrides!")
    raise RuntimeError("ERROR: Too many item_overrides!")
  file.write("%d %d %d %d %d %d  "%(entry[0],entry[1],entry[2],entry[3],entry[4], len(entry[5])))
  for item_override in entry[5]:
    add_tag_use(tag_uses,tag_item,item_override)
    file.write("%d "%(item_override))
  file.write("\n")
    
def save_mission_templates(variables,variable_uses,tag_uses,quick_strings,export_dir):
  file = open(export_dir + "mission_templates.txt","w",encoding="utf-8")
  file.write("missionsfile version 1\n")
  file.write(" %d\n"%(len(mission_templates)))
  for mission_template in mission_templates:
    file.write("mst_%s %s %d "%(convert_to_identifier(mission_template[mission_template_name_pos]),convert_to_identifier(mission_template[mission_template_name_pos]),mission_template[mission_template_flags_pos]))
    file.write(" %d\n"%(mission_template[mission_template_types_pos]))
    file.write("%s \n"%(mission_template[mission_template_desc_pos].replace(" ","_")))
    file.write("\n%d "%len(mission_template[mission_template_groups_pos]))
    for group in mission_template[mission_template_groups_pos]:
      save_mission_template_group(file,group)
    save_triggers(file,convert_to_identifier(mission_template[mission_template_name_pos]), mission_template[mission_template_triggers_pos],variables,variable_uses,tag_uses,quick_strings)
    file.write("\n")
  file.close()

def save_python_header(src_dir):
  file = open(src_dir + "ID_mission_templates.py","w",encoding="utf-8")
  for i_mission_template in range(len(mission_templates)):
    file.write("mst_%s = %d\n"%(mission_templates[i_mission_template][0],i_mission_template))
  file.close()


def processMissionTmps(context):
  print( "Exporting mission_template data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_mission_templates(variables, variable_uses, tag_uses, quick_strings,export_dir)
  save_variables(export_dir, variables, variable_uses)
  save_tag_uses(export_dir, tag_uses)
  save_quick_strings(export_dir, quick_strings)

  #print( "Finished.")

  """  
    (
      "defense_1",mtf_battle_mode,stand_fast,
      "You wait, your weapons ready, your senses alert. Some of your companions start to sing an old song, trying to forget their fear. You smile and let your thoughts wander away for a second. Then the lookout's cry shatters the melancholy: 'Enemies! They are coming'",
      [
       (2,mtef_leader_only,0,group(1),1),
       (2,mtef_no_leader,0,group(1),3),
       (0,mtef_no_leader,0,group(1),0),
       (4,mtef_enemy_party|mtef_reverse_order,0,group(2)|aif_start_alarmed,5)],
      [],
      [
        (ti_val(0), ti_val(0), ti_once, [[mission_cookie_eq,0,0],[store_mission_timer_a,1],[ge,reg(1),10],[store_enemy_count,2],[lt,reg(2),3]], [[add_reinforcements_to_entry,3,3],[reset_mission_timer_a],[increment_mission_cookie,0]]),
        (ti_val(0), ti_val(0), ti_once, [[mission_cookie_eq,0,1],[store_mission_timer_a,1],[ge,reg(1),10],[store_enemy_count,2],[lt,reg(2),3]], [[add_reinforcements_to_entry,3,3],[reset_mission_timer_a],[increment_mission_cookie,0]]),
        (ti_val(0), ti_val(0), ti_once, [[mission_cookie_eq,5,0],[store_mission_timer_b,1],[ge,reg(1),10],[store_friend_count,2],[lt,reg(2),3]], [[add_reinforcements_to_entry,2,2],[reset_mission_timer_b],[increment_mission_cookie,5]]),
        (ti_val(0), ti_val(0), ti_once, [[mission_cookie_eq,5,1],[store_mission_timer_b,1],[ge,reg(1),10],[store_friend_count,2],[lt,reg(2),3]], [[add_reinforcements_to_entry,2,2],[reset_mission_timer_b],[increment_mission_cookie,5]]),
        (ti_val(0), ti_val(0), ti_once, [[all_enemies_defeated,2]], [[set_mission_result,1] ,[finish_mission,1]]),
        (ti_val(0), ti_val(0), ti_once, [[main_hero_fallen]], [[set_mission_result,-1],[finish_mission,1]]),
      ],
    ),
     (
      "chase_1",mtf_battle_mode,chase,
      "You close up on the enemy thinking that this will be an easy victory. But as you come within shooting range, enemy fighters stop fleeing and turn to meet you. Perhaps they are experienced enough to know that they can't outrun you. Or perhaps their warrior pride triumphed over their fear. Whatever it is, these men now seem to be willing to put up a fight and your job will not be easy",
      [(0,mtef_leader_only,0,group(1),1),(0,mtef_regulars_only,0,group(1),1),(1,mtef_regulars_only|mtef_enemy_party,0,aisb_hit_run,1),(2,mtef_regulars_only|mtef_enemy_party,0,0,2),(3,mtef_regulars_only|mtef_enemy_party,0,0,1),(4,mtef_regulars_only|mtef_enemy_party,0,group(2),2)],
      [],
      [
        (ti_val(0), ti_val(0), ti_once, [[all_enemies_defeated,2]], [[set_mission_result,1] ,[finish_mission,1]]),
        (ti_val(0), ti_val(0), ti_once, [[main_hero_fallen]], [[set_mission_result,-1],[finish_mission,1]]),
      ],
    ),
    (
      "slip_siege_fight_1",mtf_battle_mode,retreat_fight,
      "You lead your retreating party through a back road which, oddly enough, seems to be completely unguarded.\
   You decide to proceed with extreme caution. Very soon, you spot a movement in the bushes ahead.\
   Not taking any chances, you send an arrow into the bushes. The arrow sinks behind leaves and immediately produces\
   a sharp cry followed by some heavy cursing. Within seconds, a score of armored warriors rush out of hiding places.",
      [(2,mtef_leader_only,0,group(1),1),(2,mtef_no_leader,0,group(1)|aif_start_alarmed,3),(3,mtef_enemy_party|mtef_reverse_order,0,aif_start_alarmed,6)],
  #    [(0,mtef_leader_only,0,group(1),1),(0,mtef_regulars_only,0,group(1),2|mtnf_const|mtnf_rand),(3,mtef_regulars_only|mtef_enemy_party,0,aif_start_alarmed,2|mtnf_const),(4,mtef_regulars_only|mtef_enemy_party,0,aif_start_alarmed,2|mtnf_const|mtnf_rand)],
      [],
      [
        (ti_val(0), ti_val(0), ti_once, [[all_enemies_defeated,3]], [[set_mission_result,1] ,[finish_mission,1]]),
        (ti_val(0), ti_val(0), ti_once, [[main_hero_fallen]], [[set_mission_result,-1],[finish_mission,1]]),
      ],
    ),
  """



from smart_module.module_system.module_scripts import *


'''
    领主收租功能
    
    领主会像玩家一样进行收租，用于军队建设，也会保存一些给城镇或村庄，用于基础维修
    
    如果据点没有主人，就会一直保存租金，直到有领主上任
'''

## 领主自动收税时间(7天收一次，和玩家一样)
lord_auto_collect_tax_interval = 24 * 7


## 村庄保留租金的比例 20%（用于村庄建设）
village_retain_ratio = 20


## 领主保留租金的比例 80%（用于军队招募和升级）
lord_retain_ratio = 100 - village_retain_ratio


lordCollectionRents = {
    "name":"LordCollectionRents",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
        领主收租功能
        领主会像玩家一样进行收租，用于军队建设，也会保存一些给城镇或村庄，用于基础维修
        如果据点没有主人，就会一直保存租金，直到有领主上任
    ''',
    "actions":[
        ("Append@simple_triggers", [
            ## 领主收税
            (lord_auto_collect_tax_interval, [
                (try_for_range, ":center_no", centers_begin, centers_end),
                    (party_get_slot, ":lord", ":center_no", slot_town_lord),
                    ## 如果领地有主人
                    (ge, ":lord", 0),
                    (party_get_slot, ":rents", ":center_no", slot_center_accumulated_rents),

                    ## 计算领主的租金(80%)
                    (store_mul, ":lord_rents", ":rents", lord_retain_ratio),
                    (val_div, ":lord_rents", 100),

                    ## 设置领主的财富
                    (troop_get_slot, ":wealth", ":lord", slot_troop_wealth),
                    (val_add, ":wealth", ":lord_rents"),
                    (troop_set_slot, ":lord", slot_troop_wealth, ":wealth"),

                    ## 计算领主的租金(20%)
                    (store_mul, ":center_rents", ":rents", village_retain_ratio),
                    (val_div, ":center_rents", 100),

                    ##领地的财富(用于村庄建设或巡逻队维护等等)
                    (party_get_slot, ":town_wealth", ":center_no", slot_town_wealth),
                    (val_add, ":town_wealth", ":center_rents"),
                    (party_set_slot, ":center_no", slot_town_wealth, ":town_wealth"),

                    ## 清空租金
                    (party_set_slot, ":center_no", slot_center_accumulated_rents, 0),

                    # (str_store_party_name,s1,":center_no"),
                    # (str_store_troop_name,s2,":lord"),
                    # (assign,reg1,":rents"),
                    # (assign,reg2,":lord_rents"),
                    # (assign,reg3,":center_rents"),
                    # (display_message,"@{s2} at {s1} get rents {reg1},add wealth is {reg2}，center add wealth is {reg3}"),
                (try_end),
            ]),
        ]),
    ],
}